/* ----------------------------------------------------------------------
   LAMMPS - Large-scale Atomic/Molecular Massively Parallel Simulator
   www.cs.sandia.gov/~sjplimp/lammps.html
   Steve Plimpton, sjplimp@sandia.gov, Sandia National Laboratories

   Copyright (2003) Sandia Corporation.  Under the terms of Contract
   DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains
   certain rights in this software.  This software is distributed under 
   the GNU General Public License.

   See the README file in the top-level LAMMPS directory.
------------------------------------------------------------------------- */

#ifndef PAIR_EAM_H
#define PAIR_EAM_H

#include "pair.h"

class PairEAM : public Pair {
 public:
  PairEAM();
  ~PairEAM();
  void init();
  void settings(int, char **);
  void set_coeff(int, char **);
  void set_coeff(char *);
  void compute(int, int);
  void write_restart(FILE *) {}
  void read_restart(FILE *) {}

 private:
  int style;                 // 0 for setfl, 1 for funcfl, -1 for not yet set
  double cutforcesq;
  int ntypes_store;

  // potential as read in from a funcfl file via read_funcfl()

  int nrho_funcfl,nr_funcfl;
  double drho_funcfl,dr_funcfl,cut_funcfl;
  double mass_funcfl;
  double *frho_funcfl,*rhor_funcfl,*zr_funcfl;

  // potential as read in from a setfl file via read_setfl()

  int ntypes_setfl,nrho_setfl,nr_setfl;
  double drho_setfl,dr_setfl,cut_setfl;
  double *mass_setfl;
  double **frho_setfl,**rhor_setfl,***z2r_setfl;

  // mapping from setfl file types to atom types

  int *map;

  // potential in funcfl form for each atom type, set by store_funcfl()

  int *nrho_one,*nr_one;
  double *drho_one,*dr_one,*cut_one;
  double **frho_one,**rhor_one,**zr_one;

  // potential in final multi-type setfl format
  // set by convert_funcfl() and store_setfl()

  int nrho,nr;
  double drho,dr;
  double **frho,**rhor;
  double ***z2r;

  // potential in spline form used for force computation
  // set by interpolate()

  double rdr,rdrho;
  double **rhor_0,**rhor_1,**rhor_2,**rhor_3,**rhor_4,**rhor_5,**rhor_6;
  double **frho_0,**frho_1,**frho_2,**frho_3,**frho_4,**frho_5,**frho_6;
  double ***z2r_0,***z2r_1,***z2r_2,***z2r_3,***z2r_4,***z2r_5,***z2r_6;

  // local functions

  void allocate();
  void pre_funcfl();
  void pre_setfl();
  void read_funcfl(char *);
  void read_setfl(char *);
  void store_funcfl(int);
  void store_setfl(int, int, int, int);
  void convert_funcfl();
  void interpolate();
  void interpolate_deallocate();
  void grab(FILE *, int, double *);
};

#endif
