/* ----------------------------------------------------------------------
   LAMMPS - Large-scale Atomic/Molecular Massively Parallel Simulator
   www.cs.sandia.gov/~sjplimp/lammps.html
   Steve Plimpton, sjplimp@sandia.gov, Sandia National Laboratories

   Copyright (2003) Sandia Corporation.  Under the terms of Contract
   DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains
   certain rights in this software.  This software is distributed under 
   the GNU General Public License.

   See the README file in the top-level LAMMPS directory.
------------------------------------------------------------------------- */

#ifndef INPUT_H
#define INPUT_H

#include "stdio.h"
#include "system.h"

class Variable;

class Input : public System {
 public:
  int me;                 // proc ID
  char *command;          // ptr to current command
  int narg;               // # of command args
  int maxarg;             // max # of args in arg
  char **arg;             // parsed args for command
  char *line,*copy;       // input line & copy of it
  char *all_arg;          // all command args concatenated as one string
  int echo_screen;        // 0 = no, 1 = yes
  int echo_log;           // 0 = no, 1 = yes
  Variable *variable;     // defined variables

  Input();
  ~Input();
  void initvar(int, char **);  // grab variable settings from command-line
  char *next();                // return next executable command

 private:
  void parse();                // parse an input text line

  int next_command();
  void cd();
  void jump();
  void clear();
  void echo();
  void log();
  void variable_command();

  void units();
  void dimension();
  void processors();
  void newton();
  void boundary();
  void atom_style();

  void lattice();
  void orient();
  void origin();
  void region();
  void create_box();
  void create_atoms();
  void delete_atoms();
  void displace_atoms();
  void delete_bonds();
  void set_type();
  void mass();
  void dipole();

  void read_data();
  void read_restart();
  void replicate();

  void pair_style();
  void bond_style();
  void angle_style();
  void dihedral_style();
  void improper_style();
  void kspace_style();

  void pair_modify();
  //  void pair_write();
  void kspace_modify();
  void dielectric();

  void pair_coeff();
  void bond_coeff();
  void angle_coeff();
  void dihedral_coeff();
  void improper_coeff();
  void special_bonds();

  void temperature();
  void temp_modify();
  void neighbor_command();
  void neigh_modify();
  void group_command();
  void timestep();
  void reset_timestep();
  void velocity();

  void fix();
  void fix_modify();
  void unfix();

  void thermo();
  void thermo_style();
  void thermo_modify();
  void dump();
  void dump_modify();
  void undump();
  void restart();
  void write_restart();
  void diagnostic();
  void undiag();

  void run_style();
  void run();
  void temper();
  void min_style();
  void minimize();
};

#endif
