/* ----------------------------------------------------------------------
   LAMMPS - Large-scale Atomic/Molecular Massively Parallel Simulator
   www.cs.sandia.gov/~sjplimp/lammps.html
   Steve Plimpton, sjplimp@sandia.gov, Sandia National Laboratories

   Copyright (2003) Sandia Corporation.  Under the terms of Contract
   DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains
   certain rights in this software.  This software is distributed under 
   the GNU General Public License.

   See the README file in the top-level LAMMPS directory.
------------------------------------------------------------------------- */

#ifndef PAIR_H
#define PAIR_H

#include "system.h"

class Pair : public System {
 public:
                                       // used by other classes
  double cutforce;                     // max cutoff for all atom pairs
  double eng_vdwl,eng_coul;            // accumulated energies
  double virial[6];                    // accumulated virial

                                       // common to all pair classes
  int allocated;                       // 0/1 = whether arrays are allocated
  int **setflag;                       // whether each i,j has been set
                                       // 0 = not set, 1 = set in data file
                                       // 2 = set in input script
  double **cutsq;                      // max cutoff sq for each atom pair

  int offset_flag,mix;                 // pair modify settings
  int ntablebits;

  Pair();
  virtual ~Pair() {}

  void modify_params(int, char **);
  void virial_compute();
  //  void write_file(int, char **);

  virtual void init() = 0;
  virtual void settings(int, char **) = 0;
  virtual void set_coeff(int, char **) = 0;
  virtual void set_coeff(char *) = 0;
  virtual void compute(int, int) = 0;
  virtual void write_restart(FILE *) = 0;
  virtual void read_restart(FILE *) = 0;

  //virtual double esingle(double, int, int) {return 0.0;}
  //virtual double esingle(double, int, int, double, double) {return 0.0;}
  //virtual double fsingle(double, int, int) {return 0.0;}
  //virtual double fsingle(double, int, int, double, double) {return 0.0;}

  virtual void compute_inner() {}
  virtual void compute_middle() {}
  virtual void compute_outer(int, int) {}
};

#endif
