/* ----------------------------------------------------------------------
   LAMMPS - Large-scale Atomic/Molecular Massively Parallel Simulator
   www.cs.sandia.gov/~sjplimp/lammps.html
   Steve Plimpton, sjplimp@sandia.gov, Sandia National Laboratories

   Copyright (2003) Sandia Corporation.  Under the terms of Contract
   DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains
   certain rights in this software.  This software is distributed under 
   the GNU General Public License.

   See the README file in the top-level LAMMPS directory.
------------------------------------------------------------------------- */

// universal classes for LAMMPS styles

#ifdef AngleInclude
#endif

#ifdef AngleClass
#endif

#ifdef AtomInclude
#include "atom_atomic.h"
#endif

#ifdef AtomClass
AtomStyle(atomic,AtomAtomic)
# endif

#ifdef BondInclude
#endif

#ifdef BondClass
#endif

#ifdef DihedralInclude
#endif

#ifdef DihedralClass
#endif

#ifdef DumpInclude
#include "dump_atom.h"
#include "dump_custom.h"
#include "dump_velocity.h"
#endif

#ifdef DumpClass
DumpStyle(atom,DumpAtom)
DumpStyle(custom,DumpCustom)
DumpStyle(velocity,DumpVelocity)
#endif

#ifdef FixInclude
#include "fix_add_force.h"
#include "fix_ave_force.h"
#include "fix_centro.h"
#include "fix_com.h"
#include "fix_drag.h"
#include "fix_enforce2d.h"
#include "fix_indent.h"
#include "fix_langevin.h"
#include "fix_line_force.h"
#include "fix_msd.h"
#include "fix_npt.h"
#include "fix_nve.h"
#include "fix_nvt.h"
#include "fix_plane_force.h"
#include "fix_respa.h"
#include "fix_rigid.h"
#include "fix_set_force.h"
#include "fix_shake.h"
#include "fix_spring.h"
#include "fix_temp_rescale.h"
#include "fix_tmd.h"
#include "fix_viscous.h"
#include "fix_volume_rescale.h"
#include "fix_wall_lj93.h"
#include "fix_wiggle.h"
#endif

#ifdef FixClass
FixStyle(addforce,FixAddForce)
FixStyle(aveforce,FixAveForce)
FixStyle(CENTRO,FixCentro)
FixStyle(com,FixCOM)
FixStyle(drag,FixDrag)
FixStyle(enforce2d,FixEnforce2D)
FixStyle(indent,FixIndent)
FixStyle(langevin,FixLangevin)
FixStyle(lineforce,FixLineForce)
FixStyle(msd,FixMSD)
FixStyle(npt,FixNPT)
FixStyle(nve,FixNVE)
FixStyle(nvt,FixNVT)
FixStyle(planeforce,FixPlaneForce)
FixStyle(RESPA,FixRespa)
FixStyle(rigid,FixRigid)
FixStyle(setforce,FixSetForce)
FixStyle(shake,FixShake)
FixStyle(spring,FixSpring)
FixStyle(temp/rescale,FixTempRescale)
FixStyle(tmd,FixTMD)
FixStyle(viscous,FixViscous)
FixStyle(volume/rescale,FixVolRescale)
FixStyle(wall/lj93,FixWallLJ93)
FixStyle(wiggle,FixWiggle)
#endif

#ifdef ImproperInclude
#endif

#ifdef ImproperClass
#endif

#ifdef KSpaceInclude
#endif

#ifdef KSpaceClass
#endif

#ifdef MinInclude
  //#include "min_cg.h"
  //#include "min_htfn.h"
#endif

#ifdef MinClass
  //MinStyle(htfn,MinCG)
  //MinStyle(htfn,MinHTFN)
#endif

#ifdef PairInclude
#include "pair_lj_cut.h"
#include "pair_lj_cut_coul_cut.h"
#include "pair_lj_cut_coul_debye.h"
#include "pair_lj_expand.h"
#include "pair_soft.h"
#endif

#ifdef PairClass
PairStyle(lj/cut,PairLJCut)
PairStyle(lj/cut/coul/cut,PairLJCutCoulCut)
PairStyle(lj/cut/coul/debye,PairLJCutCoulDebye)
PairStyle(lj/expand,PairLJExpand)
PairStyle(soft,PairSoft)
#endif

#ifdef RegionInclude
#include "region_block.h"
#include "region_cylinder.h"
#include "region_sphere.h"
#endif

#ifdef RegionClass
RegionStyle(block,RegBlock)
RegionStyle(cylinder,RegCylinder)
RegionStyle(sphere,RegSphere)
#endif

#ifdef TempInclude
#include "temp_full.h"
#include "temp_partial.h"
#include "temp_ramp.h"
#endif

#ifdef TempClass
TempStyle(full,TempFull)
TempStyle(partial,TempPartial)
TempStyle(ramp,TempRamp)
#endif

#ifdef ThermoInclude
#include "thermo_multi.h"
#include "thermo_one.h"
#endif

#ifdef ThermoClass
ThermoStyle(multi,ThermoMulti)
ThermoStyle(one,ThermoOne)
#endif

// additional style files for specific packages

  //#include "style_dipole.h"
#include "style_molecule.h"
#include "style_metal.h"
#include "style_kspace.h"
#include "style_gran.h"

#include "style_user.h"
