 # Perl script by Jessica M. Benjamin and Paul S. Crozier (e-mail: pscrozi@sandia.gov)
 # Sandia National Laboratories, August 2003
 # This program will convert CHARMM input files to LAMMPS input files
 # This script requires as input the charmm topology, parameter, restart, *.crd and
 # *.psf files, and that they be named in the input file "in_ch2lmp" in that order,
 # one line per entry. You must have charmm and lmp executables in your path.

 print " Compiling mkl.f90. \n";
 system ("f90 -O -o mkl mkl.f90");

 open (INCHRMLMP, "in_ch2lmp") or die " Can't open input file. \n";  
 $file_p = <INCHRMLMP>;
 $file_t = <INCHRMLMP>;
 $file_r = <INCHRMLMP>;
 $file_c = <INCHRMLMP>;
 $file_f = <INCHRMLMP>;

 # trim the file names

 $file_p =~ s/\s*$//;
 $file_t =~ s/\s*$//;
 $file_r =~ s/\s*$//;
 $file_c =~ s/\s*$//;
 $file_f =~ s/\s*$//;

 open (PAR, $file_p) or die " Can't open parameter file. \n";                           
 open (BOND, ">tmp_bonds");
 open (ANGLE, ">tmp_angles");
 open (DIHED, ">tmp_dihedrals");
 open (IMPROPER, ">tmp_impropers");
 open (NONBOND, ">tmp_nonbond");

 print " The parameter file '$file_p' has been opened. \n";
 
 while (<PAR>)                                  # create temp files from parameter file
 {

  $r1= substr($_, 0, 5); 
 
  if ($switch == 5)
  {
     if ($r1 =~/HBOND/)
     {
       $switch = 0;
     }
   
     elsif ($r1 =~ /NBFIX/)                  
     {
        print " NBFIX found. \n";
        $switch = 0;
     }
     
     elsif ($r1 =~ /[^a-z]/)
     { 
        print NONBOND $_;
     }
 
     else
     {     
     }
  }

  if ($switch == 4)
  {
     if ($r1 =~ /NONBO/)
     {
        $switch = 5;
     }

     else
     {
        print IMPROPER  $_;
     }
  }

  if ($switch == 3)
  {
     if ($r1 =~ /IMPRO/)
     {
        $switch = 4;
     }
     else 
     {
        print DIHED $_;
     }
  }

  if ($switch == 2) 
  {
     if ($r1 =~ /DIHED/)
     {
        $switch = 3;
     }
     else
      {
        print ANGLE $_;
      }
  }

  if ($switch == 1)
  {
      if ($r1 =~ /ANGLE/)
      {
         $switch = 2;
      }

      else
      {
         print BOND $_;
      }
  }
     
  if ($r1 =~ /BONDS/)
  {
      $switch = 1;
  }
  else
  {
  } 
 }
 
 close (DIHED);
 close (NONBOND);
 close (IMPROPER);
 close (ANGLE);
 close (BOND);
 
 open (DIHED, 'tmp_dihedrals');                          #creating FF files
 open (DIHEDFF, ">dihedrals.FF");

 while (<DIHED>)
 {
     ($m1, $m2, $m3, $m4, $m5, $m6, $m7, $m8) = split (' ', $_);

     if ($m1 =~ /^!/)
     {
        $switch=11;
     }
     elsif ($m8 =~ /!/)
     {
        print DIHEDFF  $m1. '    ';
      	print DIHEDFF  $m2. '    ';
     	print DIHEDFF  $m3. '    ';
        print DIHEDFF  $m4. '    ';
     	print DIHEDFF  $m5. '    ';
     	print DIHEDFF  $m6. '    ';
      	print DIHEDFF "$m7   \n";
     }
     elsif($_ =~ /^\n/) 
    {    	
        $switch = 11;
    }
    else
    {
    	print DIHEDFF $_;
    }
 }

 open (ANGLES, 'tmp_angles');
 open (ANGLEFF, ">angles.FF");

 while (<ANGLES>)
 {
     ($g1, $g2, $g3, $g4, $g5, $g6, $g7, $g8) = split (' ', $_);
 
     if ($g1 =~ /^!/)
     {
        $switch=11;
     }	
     elsif ($g8 =~ /!/)
     {
      	print ANGLEFF  $g1. '    ';
      	print ANGLEFF  $g2. '    ';
     	print ANGLEFF  $g3. '    ';
        print ANGLEFF  $g4. '    ';
     	print ANGLEFF  $g5. '    ';
     	print ANGLEFF  $g6. '    ';
      	print ANGLEFF  "$g7  0.0 0.0 \n";
     }
     elsif ($g6 =~ /!/)
     {
     	print ANGLEFF  $g1. '    ';
      	print ANGLEFF  $g2. '    ';
     	print ANGLEFF  $g3. '    ';
        print ANGLEFF  $g4. '    ';
     	print ANGLEFF  "$g5   0.0 0.0  \n" ;
     }
     elsif($_ =~ /^\n/) 
     {
     	$switch = 11;
     }  
     else
     {
    	print ANGLEFF $g1. '   ';
        print ANGLEFF $g2. '   ';
        print ANGLEFF $g3. '   ';
        print ANGLEFF $g4. '   ';
        print ANGLEFF $g5. '   ';
        print ANGLEFF $g6. '   ';
        print ANGLEFF $g7. '   ';
        print ANGLEFF "$g8  0.0 0.0  \n";    
     }
 }

 open (IMPROPER, 'tmp_impropers');
 open (IMPROPERFF, ">impropers.FF");
 
 while (<IMPROPER>)
 {
    ($x1, $x2, $x3, $x4, $x5, $x6, $x7, $x8) = split (' ', $_);

    if ($x1 =~ /^!/)
    {
       $switch=11;
    }
    elsif ($x8 =~ /!/)
    {
      	print IMPROPERFF  $x1. '    ';
      	print IMPROPERFF  $x2. '    ';
     	print IMPROPERFF  $x3. '    ';
        print IMPROPERFF  $x4. '    ';
     	print IMPROPERFF  $x5. '    ';
     	print IMPROPERFF  $x6. '    ';
      	print IMPROPERFF  "$x7  \n";
    }
    elsif($_ =~ /^\n/) 
    {
     	$switch = 11;
    }
    else
    {
    	print IMPROPERFF $_;
    }
 }
 
 open (BONDS, 'tmp_bonds');
 open (BONDFF, ">bonds.FF");

 while (<BONDS>)
 {
 ($y1, $y2, $y3, $y4, $y5, $y6, $y7, $y8) = split (' ', $_);

    if ($y1 =~ /^!/)
    {
        $switch=11;
    }
    elsif ($y5 =~ /!/)
    {
      	 print BONDFF  $y1. '    ';
      	 print BONDFF  $y2. '    ';
     	 print BONDFF  $y3. '    ';
         print BONDFF  "$y4   \n";
    }
    elsif($_ =~ /^\n/) 
    {
     	$switch = 11;
    }
    else
    {
    	print BONDFF $_;
    }
 }

 open (NONBOND, 'tmp_nonbond');
 open (NONBONDFF, ">nonbond.FF");

 while (<NONBOND>)
 {
    ($g1, $g2, $g3, $g4, $g5, $g6, $g7, $g8) = split (' ', $_);

    if ($g1 =~ /^!/)
    {
        $switch=11;
    }
    elsif ($g8 =~ /!/)
    {
      	print NONBONDFF  $g1. '    ';
      	print NONBONDFF  $g2. '    ';
     	print NONBONDFF  $g3. '    ';
        print NONBONDFF  $g4. '    ';
     	print NONBONDFF  $g5. '    ';
     	print NONBONDFF  $g6. '    ';
      	print NONBONDFF  "$g7  0.0 -9. 9. \n";
    }
    elsif ($g5 =~ /!/)
    {
     	print NONBONDFF  $g1. '    ';
      	print NONBONDFF  $g2. '    ';
     	print NONBONDFF  $g3. '    ';
        print NONBONDFF  "$g4  0.0 -9. 9. \n";    
    }
    elsif($_ =~ /^\n/) 
    {
     	$switch = 11;
    }
    else
    {
   	print NONBONDFF $g1. '   ';
        print NONBONDFF $g2. '   ';
        print NONBONDFF $g3. '   ';
        print NONBONDFF $g4. '   ';
        print NONBONDFF $g5. '   ';
        print NONBONDFF $g6. '   ';
        print NONBONDFF $g7. '   ';
        print NONBONDFF "$g8  0.0 -9. 9.  \n";               #last of FF files has been created    
    }
 }

 close (PAR);
 close (BONDFF);
 close (ANGLEFF);
 close (DIHEDFF);
 close (IMPROPERFF);
 close (NONBONDFF); 

 open (TOPOL, $file_t) or die " Can't open topology file\n";
 open (TYPE, ">Type.list");
 
 while(<TOPOL>)     #while loop to extract info from Topology file
 {
        ($s1, $s2, $s3, $s4, $s5) = split (' ',$_);  
	             

	if ($s1 =~ /MASS/ )
	{
           if ($s1 =~ /^!/)
 	   {
              $switch = 18;  
           }      

           elsif ($s5 =~ /!/)
           {
              print TYPE $s1 . '    ';
              print TYPE $s2 . '    ';
              print TYPE $s3 . '    ';
              print TYPE "$s4  \n"; 
           }   

	   else
           {
              print TYPE $s1 . '    ';
              print TYPE $s2 . '    ';
              print TYPE $s3 . '    ';
              print TYPE $s4 . '    '; 
              print TYPE "$s5  \n"; 
	   }
        }        
 
 	
        if ($s1 =~ /DECL/)
 	{
           $switch = 7;
        }

      
	else
        {
	   $switch = 8;
        }

 }         

 close (TOPOL);
 close (TYPE);

 print " Type.list has been created. \n";
 
 open (GETTOP, "> get_top.inp");
 print GETTOP "* Get the topology \n";
 print GETTOP "* \n";
 print GETTOP "open unit 1 read card name $file_f \n";
 print GETTOP "read psf card unit 1 \n";
 print GETTOP "close unit 1 \n";
 print GETTOP "open read card unit 1 name $file_c \n";
 print GETTOP "read coor card unit 1 \n";
 print GETTOP "print psf \n";
 print GETTOP "stop \n";
 close (GETTOP);

 print " Now running CHARMM to get psf.out . . . \n";
 system ("charmm < get_top.inp > psf.out");

 open (TOPLIST, ">top.list");
 open (PSFOUT, "psf.out");

 while (<PSFOUT>)                #Loop to modify psf.out and create top.list file
 {
   
    $e3 = substr ($_, 10, 10);  
                 

    if ($e3 =~ /BOND ARRAY/)
    {
    	$switch = 9; 
    }

    if ($switch == 9) 
    {
     	if ($e3 =~ /NON-BONDED/)
     	{ 
     		$switch = 10; 
     	}	

        else 
        {
        	print TOPLIST $_;
        }

     }

     else 
     {
		$switch = 11;
     }


 }

 close (TOPLIST);
 close (PSFOUT);

 open (SED, ">psfout.sed");
 print SED "1,/ATOM CHARACTERISTICS/d";
 print SED "\n";
 print SED '/BOND ARRAY/,$d';
 print SED "\n";
 print SED "/RESIDUE/d";
 print SED "\n";
 print SED "/GROUP/d";
 print SED "\n";
 print SED '/^$/d';
 print SED "\n";
 print SED "/^ *[A-Z]/d";
 print SED "\n";
 close (SED);

 system ('#!/bin/sh');
 system ('sed -f psfout.sed psf.out > tmpa');
 system ('grep "Number of atoms" psf.out | tail -1 >tmpb');
 system ('grep "Number of bonds" psf.out | tail -1 >>tmpb');
 system ('grep "Number of dihedrals" psf.out | tail -1 >>tmpb');
 system ('grep "A *=.*B *=.*C *=" psf.out >>tmpb');
 system ("sed -e 's/A *=//' tmpb | sed -e 's/B *=//' | sed -e 's/C *=//' >atom3.list");
 system ('cat tmpa >>atom3.list');
 system ('rm tmpa tmpb');
   
 open (RES, "$file_r") or die " Can't open restart file \n";
 open (CRD, "$file_c") or die " Can't open coordinate file \n";
 open (NEW_CRD , ">tmp.crd") or die " Can't open tmp.crd \n";

 $crystal = 0;
  
 while (<RES>)         #loop in *.res to extract x,y, and z dimensions
 {
      ($e1, $e2, $e3, $e4, $e5, $e6, $e7, $e8) =split(' ', $_);  
      if ($switch == 13)
      {
         if($crystal == 0)
         {
            $x=$e1;
            $y=$e3;
            $crystal = 1;
         } 
         else
         {
            $z=$e3;
            $switch=14;
         }
      }

     if ($e1 =~ /!CRYSTAL/)
     {
            $switch=13;
     }
     else
     {
     }
 }
 
 print " x = $x \n";
 print " y = $y \n";
 print " z = $z \n";
 $file_atom = 'atom3.list';
 open (ATOM_NEW, $file_atom);
 open (TOP_ATOM, ">top_atom"); 

 while (<ATOM_NEW>)                    #loop in atom3.list to place x, y, and z dimensions in document
 {
      ($e1, $e2, $e3,$e4, $e5, $e6, $e7, $e8) = split (' ', $_); 
       $e9 = substr ($_, 24, 1);       
 	
 	if ($e1 =~ /Number/)
	{
 	  print TOP_ATOM $_;         
        }
 
        if ($e8 =~ /impropers/)
	{
          print TOP_ATOM $x. ' ' ;
          print TOP_ATOM $y. ' ' ;
 	  print TOP_ATOM "$z  \n";    
        }

       if ($e9 =~ /\d/)
       {        
       print TOP_ATOM $_;        
       }
       else 
       {
       }   
 }
 

 for ($i=0; $i<5; $i++)          #creates five comment lines at top of CRD file
 {     		
 	print NEW_CRD "\n";             	    
 }  
 
 while (<CRD>)
 {
       $aster = substr($_, 0, 1);

       if ($aster =~ /\*/)
       {
       }
       else 
       {
         print NEW_CRD $_;
       }
 } 
   
 close (TOP_ATOM);
 close (ATOM_NEW);
 close (RES);
 close (CRD);
 close (NEW_CRD);


 system ("mv top_atom atom3.list"); 
 print " Starting up the 'make LAMMPS' FORTRAN program 'mkl'. \n";
 system ("mkl < tmp.crd"); 
 print " Done with 'mkl'. \n";
 system ("cat input.head input.tail > lammps_data");
 open (PAR, $file_p) or die " Can't open parameter file. \n"; 

 $A1; 
 $B1[1]=5;            #initializing vectors
 $B2[1]=5;
 $B3[1]=5;
 $B4[1]=5;
 $i=0;
  
 while (<PAR>)          #loop to extract atoms under NBFIX in parameter file
 {
       ($A1, $A2, $A3, $A4)= split (' ',$_);     
       if ($A1 =~ /NBFIX/)
       {
           $switch =1;    
       }       
       elsif ($A1 =~ /HBOND/)
       {
           $switch =0;      
       } 
       elsif ($switch == 1)
       {
           if ($A1 =~ /^!/)        
           {                        
           }     
           else
           {
                $B1[$i] = $A1;         #Feeds atom name into array
		$B2[$i] = $A2;         #Feeds 2nd atom name into array
		$B3[$i] = $A3;         #Feeds Emin number into array
		$B4[$i] = $A4;         #Feeds Rmin number into array
		$i++;	 	                                     
           } 		
       }
       else
       {
       }
 }

  close (PAR);        
  open (TYPES, 'types.dat') or die " Cannot open types.dat \n";
  $D1;
  $D2;

  while (<TYPES>)                          #using loop in Types.dat to find corresponding number to NBFIX atoms
  {
  	($C1, $C2, $C3, $C4) = split (' ', $_);
                for ($i=0; $i<7; $i++)
                {
        		if($C4 eq $B1[$i])
                        {
                        	$D1[$i] = $C2;	 
                        }

                        if ($C4 eq $B2[$i])
                        {
                               $D2[$i] = $C2;    
                        }
                	else
                	{
                	}
               }
 }

 close (TYPES);

 open (SHAKEN, 'shaken') or die " Cannot open shaken. \n";
 open (IN, ">in_lammps");

 print " Creating lammps input file. \n";

 print IN "units           real  \n";
 print IN "neigh_modify    delay 5 every 1   \n";

 print IN "\natom_style      full  \n";
 print IN "bond_style      harmonic \n";
 print IN "angle_style     charmm \n"; 
 print IN "dihedral_style  charmm \n";
 print IN "improper_style  harmonic \n";

 print IN "pair_style      lj/charmm/coul/long 8. 10. \n";
 print IN "pair_modify     mix arithmetic \n";
 print IN "kspace_style    pppm 1E-4 \n";

 print IN "\nread_data       lammps_data";

 for ($i=0; $i<7; $i++)             #loop that sorts pair coeff and makes coeff calculations
 {
     
        if ($D1[$i] != 0 )
	{
             $B3[$i]= -1*$B3[$i]; 
             $B4[$i]= $B4[$i]/(2**(1/6));
  
             @array = ($D1[$i], $D2[$i]);
             @sortedarray = sort{$a <=> $b} @array;     #sorted so that smallest coefficient is always first
             
             print IN "\n\npair_coeff      @sortedarray $B3[$i] $B4[$i] $B3[$i] $B4[$i]";
        } 
        else
        {
        }        
 }      
 
 print IN "\n\nvelocity        all create 0.0 12345678 \n";
 print IN "\nspecial_bonds   charmm \n";
 print IN "\nfix             1 all nve";
 print IN "\nfix             2 all shake 0.000001 500 0 m 1.0";

 while (<SHAKEN>)
 {
  print IN "$_";
 }

 close SHAKEN;

 print IN "thermo          1 \n";
 print IN "thermo_style    multi \n";
 print IN "timestep        2.0   \n";
 
 print IN "\nrun             20 \n";
     
 close IN;

 print " Done making LAMMPS input file. \n";
 print " Starting LAMMPS energy calculations. \n";

 system ("lmp < in_lammps > out_lammps");      #running input script in LAMMPS

 print " LAMMPS energy calculations are completed. \n";

 open (ZERO, '>0K.inp');
 print ZERO "* perform run that starts at 0 K \n";
 print ZERO "* \n";
 print ZERO "open unit 1 read form name $file_t \n";
 print ZERO "read rtf card unit 1 \n";
 print ZERO "open unit 1 read form name $file_p \n";
 print ZERO "read param card unit 1 \n";
 print ZERO "open unit 1 read card name $file_f \n";
 print ZERO "read psf card unit 1 \n";
 print ZERO "open read card unit 1 name $file_c \n";
 print ZERO "read coor card unit 1 \n";
 $x1 = substr($x, 0, 10);
 $x2 = substr($x, 18, 3);
 printf ZERO "set lx %sE%s \n", $x1, $x2; 
 $y1 = substr($y, 0, 10);
 $y2 = substr($y, 18, 3);
 printf ZERO "set ly %sE%s \n", $y1, $y2;  
 $z1 = substr($z, 0, 10);
 $z2 = substr($z, 18, 3);
 printf ZERO "set lz %sE%s \n", $z1, $z2; 
 print ZERO 'crystal define orthorhombic @lx @ly @lz 90.0 90.0 90.0';   
 print ZERO "\n";
 print ZERO "open unit 1 read card name cubic.cry \n";
 print ZERO "crystal read unit 1 card \n";
 print ZERO "image fixed sele all end \n";
 print ZERO "shake bonh param tol 1.0e-6 \n";
 print ZERO "dynamics leap start timestep 0.002 nstep 20 nprint 1 ntrfrq 20 - \n";
 print ZERO "  firstt 0 cutnb 12 cutim 12 ctofnb 10. ctonnb 8.0 - \n";
 print ZERO "  Ewald kappa 0.320 pmEwald order 4 fftx 64 ffty 64 fftz 64 \n";
 print ZERO "stop \n";
 close (ZERO);

 open (CUBIC, '>cubic.cry');
 print CUBIC "* CRYSTAL CARD \n";
 print CUBIC "*  DATE:     8/27/ 3     16:13: 6      CREATED BY USER: crozier \n";
 print CUBIC "* \n";
 print CUBIC " SYMMETRY \n";
 print CUBIC " (X,Y,Z)  \n";                                
 print CUBIC " END \n";
 print CUBIC " \n";
 print CUBIC " IMAGES \n";
 print CUBIC " ! Operation     A     B     C \n";
 print CUBIC "           1    -1    -1    -1 \n";
 print CUBIC "           1    -1     0    -1 \n";
 print CUBIC "           1    -1     1    -1 \n";
 print CUBIC "           1     0    -1    -1 \n";
 print CUBIC "           1     0     0    -1 \n";
 print CUBIC "           1     0     1    -1 \n";
 print CUBIC "           1    -1    -1     0 \n";
 print CUBIC "           1    -1     0     0 \n";
 print CUBIC "           1    -1     1     0 \n";
 print CUBIC "           1     0    -1     0 \n";
 print CUBIC "           1     0     1     0 \n";
 print CUBIC "           1    -1    -1     1 \n";
 print CUBIC "           1    -1     0     1 \n";
 print CUBIC "           1    -1     1     1 \n";
 print CUBIC "           1     0    -1     1 \n";
 print CUBIC "           1     0     0     1 \n";
 print CUBIC "           1     0     1     1 \n";
 print CUBIC "           1     1     1     1 \n";
 print CUBIC "           1     1     0     1 \n";
 print CUBIC "           1     1    -1     1 \n";
 print CUBIC "           1     1     1     0 \n";
 print CUBIC "           1     1     0     0 \n";
 print CUBIC "           1     1    -1     0 \n";
 print CUBIC "           1     1     1    -1 \n";
 print CUBIC "           1     1     0    -1 \n";
 print CUBIC "           1     1    -1    -1 \n";
 print CUBIC " END \n";
 close (CUBIC);

 print " Starting CHARMM energy calculations. \n";

 system ("charmm < 0K.inp > out_charmm");        #running input script in CHARMM
 
 print " CHARMM energy calculations are completed. \n";

 print "\n Opening CHARMM output. \n";

 open (CHARMM, 'out_charmm');

 print " For time step 0 in CHARMM \n";

 $i = 0;
 
 while (<CHARMM>)            #opening CHARMM output and calculating energies
 {
 
	($b1, $b2, $b3, $b4, $b5, $b6, $b7)= split (' ', $_);
      
        $q1 = substr ($_, 0, 5);
        $q2 = substr ($_,12, 2); 
        $q3 = substr ($_,27,13);
        $q4 = substr ($_,68,11);
        $q5 = substr ($_,16,11);
        $q6 = substr ($_,27,13);
        $q7 = substr ($_,17,10);
        $q8 = substr ($_,27,13);
        $q9 = substr ($_,41,12);
      
	if ($b1 =~ /DYNA>/)
        {
        	 print "\n Time Step $q2  \n";
        	 print " Total Energy = $q3   \n";
        	 print " Temperature = $q4 \n";

         	 $tot_energy[$i] = $q3;
                 $temp[$i] = $q4;       
        }

        elsif ($b2 =~ /INTERN>/ && $b1 =~ /DYNA/)
        {	
		 print " Bond Energy = $b3 \n";
                 $b8 = $b4 + $b5;                 #addition of angle and Urey_b terms to get angle energy
                 print " Angle Energy = $b8 \n";
                 print " Dihedral Energy = $b6 \n";
                 print " Improper Energy = $b7 \n";
           
                 $bond_energy[$i] = $b3;
                 $angle_energy[$i] = $b8;
                 $dihedral_energy[$i] = $b6;
                 $improper_energy[$i] = $b7;                
        } 
          
        elsif ($b1 =~ /DYNA/ && $b2 =~ /EXTERN>/)
        {
                 $b10 = $q6;
                 $b15 = $q5;  
                 print " Primary Cell Nonbond Energy = $b15 \n";  
                 $nonbond_energy[$i]= $b15;          
        }

        elsif ($b1 =~ /DYNA/ && $b2 =~ /IMAGES>/)
        {
                 $b11 = $b4;
                 $b16 = $b3 + $b15;
                 print " Image Nonbond Energy = $b3 \n";
                 $nonbond_energy[$i]= $b16;             
        }

        elsif ($b1 =~ /DYNA/ && $b2=~ /EWALD>/)
        { 
                 $b12 = $q7 +$q8 +$q9; 
                 $b13= $b10 + $b11;
                 $b14= $b13 + $b12;
                 print " Coulombic Energy= $b14  \n";
                 $coulombic_energy[$i] = $b14;
                 $i++;                
        }  
        else
        {
        } 
 }
 
 open (OUTLMP, 'out_lammps');

 print "\n Opening LAMMPS output. \n";

 $j = 0;
 
 while (<OUTLMP>)  #opening LAMMPS output and getting energies
 {
       ($u1, $u2, $u3, $u4, $u5, $u6, $u7, $u8, $u9, $u10) = split(' ', $_);
  
       if ($_ =~ /Step/)
       {
       	        $k = substr ($_, 13, 20);
                print "\n $k  \n";
       }  

 	elsif ($u1 =~ /Total/ && $u2 =~ /E/ && $u3 =~ /=/)
        {
                print " Total Energy = " . "$u4 \n";
                print " Temperature =  " . "$u10 \n";
                $Total_Energy[$j] = $u4;
                $Temperature[$j] = $u10;          
        }
 
        elsif ($u1 =~ /Total/ && $u2 =~ /PE=/ )
        {
        	print ' Bond energy = ' . "$u6 \n";
                $Bond_Energy[$j] = $u6;
                print ' Angle energy = ' ."$u9 \n";
                $Angle_Energy[$j] = $u9;
        }

        elsif ($u1 =~ /E_dihed/) 
        {
		print ' Dihedral Energy = ' . "$u3 \n";
                $Dihed_Energy[$j] = $u3;
                print ' Improper Energy =' . "$u6 \n";
                $Improper_Energy[$j] = $u6;
                print ' Nonbond Energy = ' . "$u9 \n";
                $Nonbond_Energy[$j] = $u9;        
        }


        elsif ($u1=~ /E_coul/)
        {
        	$u11 = $u3 + $u6;                 #addition of E_coul & E_long to get coulombic term
                print ' Coulombic Energy = ' ."$u11 \n";
                $Coulombic_Energy[$j] = $u11;
                $j++;
        }
 }

        print "\n";
 
	for( $i=0; $i <= 20; $i++)
        {     
             print "\n For time step $i: \n";              
             if ($temp[$i] > 0) 
             {           
               $Diff_Temp[$i]=abs((($temp[$i]-$Temperature[$i]))/$temp[$i]) * 100;  #use variable q to avoid dividing by 0 in time step 0.
               print "  Difference in Temperature is $Diff_Temp[$i]%\n";
             }
             else 
             {
               print "                Temperature is $temp[$i]\n";
             }
             $Diff_Energy[$i]=abs((($tot_energy[$i]-$Total_Energy[$i]))/$tot_energy[$i]) * 100;   #calculating percentage differences
             print "  Difference in Total Energy is $Diff_Energy[$i]%\n";
             $Diff_Bond[$i]=abs((($bond_energy[$i]-$Bond_Energy[$i]))/$bond_energy[$i]) *100;
             print "  Difference in Bond Energy is $Diff_Bond[$i]%\n";
             $Diff_Angle[$i]=abs((($angle_energy[$i]-$Angle_Energy[$i]))/$angle_energy[$i]) *100;
             print "  Difference in Angle Energy is $Diff_Angle[$i]%\n";
             $Diff_Dihed[$i]=abs((($dihedral_energy[$i]-$Dihed_Energy[$i]))/$dihedral_energy[$i]) *100;
             print "  Difference in Dihedral Energy is $Diff_Dihed[$i]%\n";
             $Diff_Improper[$i]=abs((($improper_energy[$i]-$Improper_Energy[$i]))/$improper_energy[$i])*100;
             print "  Difference in Improper Energy is $Diff_Improper[$i]% \n";
             $Diff_Nonbond[$i]=abs((($nonbond_energy[$i]-$Nonbond_Energy[$i]))/$nonbond_energy[$i]) *100;
             print "  Difference in Nonbond Energy is $Diff_Nonbond[$i]%\n";
             $Diff_Coulombic[$i]=abs((($coulombic_energy[$i]-$Coulombic_Energy[$i]))/$coulombic_energy[$i])*100;
             print "  Difference in Coulombic Energy is $Diff_Coulombic[$i]% \n"; 
	}
       
# delete unneeded files
 
 &garbage;

# we're done

 print "\n Done with CHARMM to LAMMPS conversion 'chrm2lmp.pl' perl script. \n";

 exit;


 sub garbage {

 unlink("GLOBAL.mod") or " Cannot unlink GLOBAL.mod: $!";
 unlink("mkl") or " Cannot unlink mkl: $!";

 unlink("psfout.sed") or " Cannot unlink psfout.sed: $!";
 unlink("psf.out") or " Cannot unlink psf.out: $!";

 unlink("tmp_bonds") or " Cannot unlink tmp_bond: $!";
 unlink("tmp_angles") or " Cannot unlink tmp_angle: $!";
 unlink("tmp_dihedrals") or " Cannot unlink tmp_dihed: $!";
 unlink("tmp_impropers") or " Cannot unlink tmp_improper: $!";
 unlink("tmp_nonbond") or " Cannot unlink tmp_nonbond: $!";

 unlink("angles.FF") or " Cannot unlink angles.FF: $!";
 unlink("bonds.FF") or " Cannot unlink bonds.FF: $!";
 unlink("dihedrals.FF") or " Cannot unlink dihedrals.FF: $!";
 unlink("impropers.FF") or " Cannot unlink impropers.FF: $!";
 unlink("nonbond.FF") or " Cannot unlink nonbond.FF: $!";

 unlink("atomsFF.dat") or " Cannot unlink atomsFF.dat: $!";
 unlink("anglesFF.dat") or " Cannot unlink anglesFF.dat: $!";
 unlink("bondsFF.dat") or " Cannot unlink bondsFF.dat: $!";
 unlink("dihedralsFF.dat") or " Cannot unlink dihedralsFF.dat: $!";
 unlink("impropersFF.dat") or " Cannot unlink impropersFF.dat: $!";
 unlink("nonbondFF.dat") or " Cannot unlink nonbondFF.dat: $!";

 unlink("atom3.list") or " Cannot unlink atom3.list: $!";
 unlink("tmp.crd") or " Cannot unlink tmp.crd: $!";
 unlink("top.list") or " Cannot unlink top.list: $!";
 unlink("types.dat") or " Cannot unlink types.dat: $!";
 unlink("Type.list") or " Cannot unlink Type.list: $!";

 unlink("shaken") or " Cannot unlink shaken: $!";
 unlink("input.head") or " Cannot unlink input.head: $!";
 unlink("input.tail") or " Cannot unlink input.tail: $!";

 unlink("get_top.inp") or " Cannot unlink get_top.inp: $!";
 unlink("0K.inp") or " Cannot unlink 0K.inp: $!";
 unlink("cubic.cry") or " Cannot unlink cubic.cry: $!";

 unlink("log.lammps") or " Cannot unlink log.lammps: $!";
 unlink("out_lammps") or " Cannot unlink out_lammps: $!";
 unlink("out_charmm") or " Cannot unlink out_charmm: $!";

 }
