! -------------------------------------------------------------------------
! Code converts CHARMM input to LAMMPS input
! Modified from mklammps code by Mark Stevens
! Paul Crozier, SNL, 2002
!
! input files:
!             stdio = *.crd 
!             atom3.list 
!             top.list            topology file
!             Type.list         
!             nonbond.FF        
!             bonds.FF         
!             angles.FF
!             dihedrals.FF
!             impropers.FF
! output files
!             input.head          cp input.head input.lammps 
!             input.tail          cat input.tail >>input.lammps
!             types.dat
! -------------------------------------------------------------------------

module global

integer, parameter :: nmax=276
integer, parameter :: maxatom = 60000
integer, parameter :: maxbonds = 60000
integer, parameter :: maxangles = 70000
integer, parameter :: maxdihedrals = 90000
integer, parameter :: maximpropers = 50000

integer natoms, maxtypes, nbonds, nangles, ndihedrals, nimpropers, nonbond
integer ntypes, columns, natomcrd
integer nbonds_top, nangles_top, ndihedrals_top, nimpropers_top
integer unneeded_bonds,ndihedrals_actual
integer i, j, k, m, icount, i0, i1, i2, i3, i4, jCR
integer atomno(nmax), atomno0
integer inonbondatom(nmax)
integer nbondtypes, nangletypes, ntorsiontypes
integer ndihedraltypes, nimpropertypes
integer rows, index
integer moltype, atomtype(maxatom), ndum, atomtype0
integer iflag, ime, i1used, i2used, i3used, i4used, ia, iangle, iprod
integer j1, j2, j3, j4, istop
integer iused(4), iii(4), HT_type, i14, idihedral, iimproper
integer inonbondatom(nmax)
integer bond1(maxbonds), bond2(maxbonds), bondtype(maxbonds)
integer angle1(maxangles), angle2(maxangles), angle3(maxangles)
integer angletype(maxangles)
integer dihedral1(maxdihedrals),dihedral2(maxdihedrals)
integer dihedral3(maxdihedrals), dihedral4(maxdihedrals)
integer dihedraltype(maxdihedrals), dihedral_list(4,maxdihedrals)
integer dihedral14(maxdihedrals)
integer beginX, noX, npossible
integer improper(4,maximpropers)
integer improper1(maximpropers),improper2(maximpropers)
integer improper3(maximpropers), improper4(maximpropers)
integer impropertype(maximpropers)
integer atom_CHARMM_num_here(nmax)
integer type_CHARMM2me(nmax),type_CHARMM2used(nmax)

real mass0, mass(nmax), mass1(nmax), ignore
real xbox,ybox,zbox,xbox2,ybox2,zbox2
real tmp
real x, y, z, q(maxatom)

character(16) work
character(80) inputline
character(4) char1
character(3) residue_name
character(4) atom_name0, atom_name(maxatom),d1,d2,d3,d4,b1,b2
character(4) atom_CHARMM_name(nmax), name(nmax)

character(4), allocatable :: angleatom1(:), angleatom2(:), angleatom3(:)
character(4), allocatable :: bondatom1(:), bondatom2(:)
character(4), allocatable :: dihedatom1(:), dihedatom2(:)
character(4), allocatable :: dihedatom3(:), dihedatom4(:)
character(4), allocatable :: improperatom1(:), improperatom2(:)
character(4), allocatable :: improperatom3(:), improperatom4(:)
character(4), allocatable :: nonbondatom(:)
character(4), allocatable :: type_used_CHARMM_name(:) 

integer, allocatable :: iangleatom1(:), iangleatom2(:)
integer, allocatable :: iangleatom3(:)
integer, allocatable :: ibondatom1(:), ibondatom2(:)
integer, allocatable :: idihedatom1(:), idihedatom2(:)
integer, allocatable :: idihedatom3(:), idihedatom4(:)
integer, allocatable :: n_dihed(:)  ! dihedral FF phase number
integer, allocatable :: iimproperatom1(:), iimproperatom2(:)
integer, allocatable :: iimproperatom3(:), iimproperatom4(:)
integer, allocatable :: improperXflag(:)
integer, allocatable :: ignored(:)  ! improper ignored number
integer, allocatable :: mytype2used(:), type_used2me(:) 
integer, allocatable :: atom_CHARMM_no(:)
integer, allocatable :: bond_matrix(:,:)
integer, allocatable :: angle_matrix(:,:), dihedral_maybe(:,:)
integer, allocatable :: dihedraltype14(:,:),dihedralmap(:)
integer, allocatable :: improper_matrix(:,:)
integer, allocatable :: bondflag(:), bondFFptr(:)
integer, allocatable :: angleflag(:), angleFFptr(:)
integer, allocatable :: ndihedral(:), dihedral14FF(:)
integer, allocatable :: improperflag(:)

real, allocatable :: k_angle(:), thangle(:), k_UB(:), r_UB(:)
real, allocatable :: kbond(:), rbond(:)  ! bond FF spring const. and length
real, allocatable :: k_dihed(:), dangle(:)  ! dihedral FF spring const. and angle
real, allocatable :: k_improper(:), ang_improper(:) ! improper FF spring const. and angle
real, allocatable :: eps(:), sigma(:)
real, allocatable :: eps14(:), sigma14(:)

end module

! -------------------------------------------------------------------------
! -------------------------------------------------------------------------

program mkl
use global
implicit none

integer intread
real realread

! Type.list contains list of atom names used in FF
open(1,file='Type.list',status='old')
open(2,file='atomsFF.dat',status='unknown')

write(2,'("My no. name CHARMM no.")')

!read number of atoms, bonds, etc.
icount = 0
do
  read(1,200,end=100,err=100) inputline    ! Reads in whole line of 80 characters
  m=1
  call strread(inputline,m,char1)
  atomno0 = intread(inputline,m)
  call strread(inputline,m,atom_name0)
  mass0 = realread(inputline,m)
  if(char1 == 'MASS') then
    icount = icount + 1
    atomno(icount) = atomno0
    atom_name(icount) = atom_name0
    mass(icount) = mass0
    write(2,300) icount, atom_name0, atomno0
  end if
end do
100 continue
maxtypes = icount
write(6,*) maxtypes, 'atom types.'
200 format(a)
300 format(i6,1x,a4,i6)

close(1)
close(2)

! angles.FF is a CHARMM bond FF file
open(3,file='angles.FF',status='old')

nangles = 0
do
  read(3,*,end=400,err=400) inputline
  nangles = nangles + 1
end do
400 continue
rewind 3
write(6,*) nangles, 'angles.'

allocate (angleatom1(nangles),angleatom2(nangles),angleatom3(nangles))
allocate (k_angle(nangles),thangle(nangles),k_UB(nangles))
allocate (r_UB(nangles))
allocate (iangleatom1(nangles),iangleatom2(nangles),iangleatom3(nangles))

do i=1,nangles
  read(3,*) angleatom1(i),angleatom2(i),angleatom3(i), &
              k_angle(i),thangle(i),k_UB(i),r_UB(i)
end do

! Convert bond FF pairs to list based on atom numbers.
do i=1,nangles
  d1 = angleatom1(i)
  d2 = angleatom2(i)
  d3 = angleatom3(i)
  i1 = 0
  i2 = 0
  i3 = 0
  if( d1 == 'X   ') then
    i1 = -1
  else
    do j=1,maxtypes
      if(d1 == atom_name(j)) i1 = j
    end do
  end if
  if( d2 == 'X   ') then
    i2 = -1
  else
    do j=1,maxtypes
      if(d2 == atom_name(j)) i2 = j
    end do
  end if
  if( d3 == 'X   ') then
    i3 = -1
  else
    do j=1,maxtypes
      if(d3 == atom_name(j)) i3 = j
    end do
  end if
  if( i1 == 0 .OR. i2 == 0 .OR. i3 == 0 ) then
    write(6,*) 'ERROR: angle without matching names.'
    write(6,*) 'FF angle names: ', i,' ',d1,' ',d2,' ',d3
    stop
  else
    iangleatom1(i) = i1
    iangleatom2(i) = i2
    iangleatom3(i) = i3
  end if
end do

! OUTPUT
open(4,file='anglesFF.dat')
do i=1,nangles
  write(4,500) angleatom1(i),angleatom2(i),angleatom3(i),       & 
               k_angle(i),thangle(i),k_UB(i),r_UB(i),       &
              iangleatom1(i),iangleatom2(i),iangleatom3(i)
end do
500 format(3(a4,1x,),4(f10.4,1x,),3(1x,i5))
close(4)

! bonds.FF is a CHARMM bond FF file
open(7,file='bonds.FF',status='old')

nbonds = 0
do
  read(7,*,end=600,err=600) inputline
  nbonds = nbonds + 1
end do
600 continue
rewind 7
write(6,*) nbonds, 'bonds.'

allocate (bondatom1(nbonds),bondatom2(nbonds))
allocate (kbond(nbonds),rbond(nbonds))
allocate (ibondatom1(nbonds),ibondatom2(nbonds))

do i=1,nbonds
  m=1
  read(7,200) inputline
  call strread(inputline,m,bondatom1(i))
  call strread(inputline,m,bondatom2(i))
  kbond(i) =  realread(inputline,m)
  rbond(i) =  realread(inputline,m)
end do

! Convert bond FF pairs to list based on atom numbers.
do i=1,nbonds
  b1 = bondatom1(i)
  b2 = bondatom2(i)
  i1 = 0
  i2 = 0
  do j=1,maxtypes
    if(b1 == atom_name(j)) i1 = j
  end do
  do j=1,maxtypes
    if(b2 == atom_name(j)) i2 = j
  end do
  if( i1 == 0 .OR. i2 == 0) then
    write(6,*) 'ERROR: bond pair without matching names.'
    write(6,*) 'FF bond pair names: ', b1,b2
    stop
  else
    ibondatom1(i) = i1
    ibondatom2(i) = i2
  end if
end do

! OUTPUT
open(8,file='bondsFF.dat',status='unknown')
do i=1,nbonds
  write(8,700) bondatom1(i),bondatom2(i),kbond(i),rbond(i),ibondatom1(i), &
              ibondatom2(i)
end do
700 format(a4,1x,a4,1x,f9.4,1x,f9.4,1x,i9,1x,i9)
close(8)

! dihedrals.FF is a CHARMM bond FF file
open(9,file='dihedrals.FF',status='old')

ndihedrals = 0
do
  read(9,*,end=800,err=800) inputline
  ndihedrals = ndihedrals + 1
end do
800 continue
rewind 9
write(6,*) ndihedrals, 'dihedrals.'

allocate (dihedatom1(ndihedrals), dihedatom2(ndihedrals))
allocate (dihedatom3(ndihedrals), dihedatom4(ndihedrals))
allocate (idihedatom1(ndihedrals), idihedatom2(ndihedrals))
allocate (idihedatom3(ndihedrals), idihedatom4(ndihedrals))
allocate (n_dihed(ndihedrals))
allocate (k_dihed(ndihedrals), dangle(ndihedrals))

do i=1,ndihedrals
  read(9,*) dihedatom1(i),dihedatom2(i),dihedatom3(i),dihedatom4(i), &
              k_dihed(i),n_dihed(i),dangle(i)
end do

! Convert bond FF pairs to list based on atom numbers.
do i=1,ndihedrals
  d1 = dihedatom1(i)
  d2 = dihedatom2(i)
  d3 = dihedatom3(i)
  d4 = dihedatom4(i)
  i1 = 0
  i2 = 0
  i3 = 0
  i4 = 0
  if( d1 == 'X   ') then
    i1 = -1
  else
    do j=1,maxtypes
      if(d1 == atom_name(j)) i1 = j
    end do
  end if
  if( d2 == 'X   ') then
    i2 = -1
  else
    do j=1,maxtypes
      if(d2 == atom_name(j)) i2 = j
    end do
  end if
  if( d3 == 'X   ') then
    i3 = -1
  else
    do j=1,maxtypes
      if(d3 == atom_name(j)) i3 = j
    end do
  end if
  if( d4 == 'X   ') then
    i4 = -1
  else
    do j=1,maxtypes
      if(d4 == atom_name(j)) i4 = j
    end do
  end if
  if( i1 == 0 .OR. i2 == 0 .OR. i3 == 0 .OR. i4 == 0) then
    write(6,*) 'ERROR: dihedral without matching names.'
    write(6,*) 'FF dihedral names: ', i,' ',d1,d2,d3,d4
    stop
  else
    idihedatom1(i) = i1
    idihedatom2(i) = i2
    idihedatom3(i) = i3
    idihedatom4(i) = i4
  end if
end do

! OUTPUT
! Put cases with X-*-*-X at end
open(10,file='dihedralsFF.dat',status='unknown')
! don't print out X containing cases
do i=1,ndihedrals
  if(idihedatom1(i) /= -1) &	
  write(10,900) dihedatom1(i),dihedatom2(i),dihedatom3(i),dihedatom4(i), & 
               k_dihed(i),n_dihed(i),dangle(i), &
              idihedatom1(i),idihedatom2(i),idihedatom3(i),idihedatom4(i)
end do
! print out only X containing cases
do i=1,ndihedrals
  if(idihedatom1(i) == -1) &
  write(10,900) dihedatom1(i),dihedatom2(i),dihedatom3(i),dihedatom4(i), & 
               k_dihed(i),n_dihed(i),dangle(i), &
               idihedatom1(i),idihedatom2(i),idihedatom3(i),idihedatom4(i)
end do

900 format(4(a4,1x,),f10.4,1x,i2,1x,f8.3,4(1x,i5))
close(10)

! impropers.FF is a CHARMM bond FF file
open(11,file='impropers.FF',status='old')

nimpropers = 0
do
  read(11,*,end=1000,err=1000) inputline
  nimpropers = nimpropers + 1
end do
1000 continue
rewind 11
write(6,*) nimpropers, 'impropers.'

allocate (improperatom1(nimpropers), improperatom2(nimpropers))
allocate (improperatom3(nimpropers), improperatom4(nimpropers))
allocate (iimproperatom1(nimpropers), iimproperatom2(nimpropers))
allocate (iimproperatom3(nimpropers), iimproperatom4(nimpropers))
allocate (improperXflag(nimpropers))
allocate (ignored(nimpropers))
allocate (k_improper(nimpropers), ang_improper(nimpropers)) 

! Note 2nd FF parameter is ignored.
do i=1,nimpropers
  read(11,*) improperatom1(i),improperatom2(i), &
            improperatom3(i),improperatom4(i), &
              k_improper(i),ignored(i),ang_improper(i)
end do

! Convert bond FF pairs to list based on atom numbers.
! improperXflag flags quads containing an X
do i=1,nimpropers
  improperXflag(i) = 0
  d1 = improperatom1(i)
  d2 = improperatom2(i)
  d3 = improperatom3(i)
  d4 = improperatom4(i)
  i1 = 0
  i2 = 0
  i3 = 0
  i4 = 0
  if( d1 == 'X   ') then
    i1 = -1
    improperXflag(i) = 1
  else
    do j=1,maxtypes
      if(d1 == atom_name(j)) i1 = j
    end do
  end if
  if( d2 == 'X   ') then
    i2 = -1
    improperXflag(i) = 1
  else
    do j=1,maxtypes
      if(d2 == atom_name(j)) i2 = j
    end do
  end if
  if( d3 == 'X   ') then
    i3 = -1
    improperXflag(i) = 1
  else
    do j=1,maxtypes
      if(d3 == atom_name(j)) i3 = j
    end do
  end if
  if( d4 == 'X   ') then
    i4 = -1
    improperXflag(i) = 1
  else
    do j=1,maxtypes
      if(d4 == atom_name(j)) i4 = j
    end do
  end if
  if( i1 == 0 .OR. i2 == 0 .OR. i3 == 0 .OR. i4 == 0) then
    write(6,*) 'ERROR: improper without matching names.'
    write(6,*) 'FF improper names: ', i,' ',d1,d2,d3,d4
    stop
  else
    iimproperatom1(i) = i1
    iimproperatom2(i) = i2
    iimproperatom3(i) = i3
    iimproperatom4(i) = i4
  end if
end do

! OUTPUT
open(12,file='impropersFF.dat',status='unknown')
! Do not print ignored quantity
! Put cases containing an X at end; tend to only have *-X-X-*, but don't 
! require that here.
do i=1,nimpropers
! don't print out X containing cases
  if( improperXflag(i) /= 1) &
  write(12,1100) improperatom1(i),improperatom2(i), &
                 improperatom3(i),improperatom4(i), & 
                 k_improper(i),ang_improper(i), &
                 iimproperatom1(i),iimproperatom2(i), &
                 iimproperatom3(i),iimproperatom4(i)
end do
! print out X containing cases
do i=1,nimpropers
  if( improperXflag(i) == 1) &
  write(12,1100) improperatom1(i),improperatom2(i), &
                 improperatom3(i),improperatom4(i), & 
                 k_improper(i),ang_improper(i), &
                 iimproperatom1(i),iimproperatom2(i), &
                 iimproperatom3(i),iimproperatom4(i)
end do

1100 format(4(a4,1x,),f10.4,1x,f8.3,4(1x,i5))
close(12)

! nonbond.FF is a CHARMM bond FF file
open(13,file='nonbond.FF',status='old')

nonbond = 0
do
  read(13,*,end=1200,err=1200) inputline
  nonbond = nonbond + 1
end do
1200 continue
rewind 13
write(6,*) nonbond, 'nonbonds.'

allocate (nonbondatom(maxtypes))
allocate (eps(maxtypes), sigma(maxtypes))
allocate (eps14(maxtypes), sigma14(maxtypes))

do i=1,nonbond
  read(13,*) nonbondatom(i),ignore, eps(i),sigma(i), ignore,eps14(i),sigma14(i)
  eps(i) = - eps(i)
  if(eps14(i) == -9.) then
    eps14(i) = eps(i)
    sigma14(i) = sigma(i)
  else
    eps14(i) = -eps14(i)
  end if
end do

! Assign to each type nonbond parameters
! First must find index of CR* in nonbondFF
jCR=0
do j=1,nonbond
  b1 = nonbondatom(j)
  if(b1(1:3) == 'CR*') jCR = j
end do
if(jCR == 0) then
    write(6,*) 'Warning: CR* missing in nonbondFF.'
else
!    write(6,*) 'jCR = ', jCR
end if

do i=1,maxtypes
  b1 = atom_name(i)
  i1 = 0
  do j=1,nonbond
    if(b1 == nonbondatom(j)) then
      i1 = j
    else if (b1(1:2) == 'CR') then
      i1 = jCR
    end if
  end do
  if( i1 == 0 ) then
    write(6,*) 'ERROR: atom without matching name.', i
    write(6,*) 'FF atom name: ', b1
    stop
  else
    inonbondatom(i) = i1
  end if
end do

! OUTPUT
open(14,file='nonbondFF.dat',status='unknown')
do i=1,maxtypes
  j = inonbondatom(i)
  write(14,1300) i,atom_name(i),eps(j),sigma(j),eps14(j),sigma14(j),mass(i),j
end do
1300 format(i4,1x,a4,1x,2(f9.5,1x,f9.5,1x),f9.5,1x,i9)
close(14)

write(6,*) 'Done with making *.dat files.'

call build_input

write(6,*) 'Done.'

end program mkl

! -------------------------------------------------------------------------

subroutine build_input
use global
implicit none

integer intread
real realread

! Allocate memory for arrays

allocate (mytype2used(maxtypes), type_used2me(maxtypes))
allocate (atom_CHARMM_no(maxtypes))
allocate (bond_matrix(maxtypes,maxtypes))
allocate (angle_matrix(3,nangles), dihedral_maybe(5,ndihedrals))
allocate (dihedraltype14(0:2,ndihedrals),dihedralmap(ndihedrals))
allocate (improper_matrix(4,nimpropers))
allocate (bondflag(nbonds), bondFFptr(nbonds))
allocate (angleflag(nangles), angleFFptr(nangles))
allocate (ndihedral(3*ndihedrals), dihedral14FF(3*ndihedrals))
allocate (improperflag(nimpropers))
allocate (type_used_CHARMM_name(maxtypes))

! Now ready to build LAMMPS input file

!      CHARMM index    my index                  used index
!      ------------    --------                  ----------
!      atom_CHARMM_no
!      atomtype                                  atomtype
!      1               1                         1
!      ...             ...                       ...
!      276             102                       ntypes
! pointer arrays:
!       |->type_CHARMM2me          type_used2me<-|
!                        |-> mytype2used ->.....->
!       |-< type_used2CHARMM  -<-<-<-<-<-<-<-<-<-<   DOES NOT EXIST.
!       |-> type_CHARMM2used  ->->->->->->->->->->


! *** IMPORTANT
!     The force-field files *FF.dat use MY INDEX.

!Initialize
! mass = 0 => atom index not yet used
do i=1,nmax
  mass1(i) = 0.0
  name(i) = 'NONE'
  type_CHARMM2used(i) = 0                   !initialize array for use later
end do
bond_matrix = 0

! atoms.dat contains list of : My index; CHARMM atom name; CHARMM index.
open(15,file='atomsFF.dat',status='old')
read(15,*)                               ! read header line
do i=1,maxtypes                                     
  read (15,*) icount,atom_CHARMM_name(i), atom_CHARMM_no(i)
  type_CHARMM2me(atom_CHARMM_no(i)) = icount
  mytype2used(i) = 0                    ! initialize
end do
close(15)

!initialize flag to 0
do i=1,nmax
  atom_CHARMM_num_here(i) = 0
end do

!.............................................................
!read atom3.list
!get list of CHARMM atom numbers used
!create compact list of atom types
open(16,file='atom3.list',status='old')

!read number of atoms, bonds, etc.
!NOTE: This ASSUMES a specific format for the first 3 lines!!!!!!!!!!!!!!!!!!!!!
! MJS Feb 13, 01 read(2,*)
read(16,1400) natoms, ndum
read(16,1400) nbonds_top, nangles_top
read(16,1400) ndihedrals_top, nimpropers_top    
read(16,*) xbox,ybox,zbox                  
1400 format(34x,i9, 25x, i9)               

xbox2 = 0.5*xbox                        
ybox2 = 0.5*ybox
zbox2 = 0.5*zbox

!flag each atomtype that exists
icount = 0
iflag = 0
do i=1,natoms
  read(16,*) index, atom_name(i), q(i), atomtype(i), i1, i2, mass0
  atom_CHARMM_num_here(atomtype(i)) = 1

  atomtype0 = atomtype(i)
  if(mass1(atomtype0).eq.0.0) then
    mass1(atomtype0) = mass0
  else
    if(mass1(atomtype0).ne.mass0) iflag = 1
    if(iflag.eq.1) then
      icount = icount + 1
      write(6,*) 'Mismatch! ', i,atomtype0,mass1(atomtype0)
      write(6,1500) atom_name(i),mass0
    end if
  end if
  if(icount.eq.10) stop
1500 format(3x,a4,2x,f8.4,2x,f11.5)

end do
close(16)

! NONBOND NONBOND NONBOND NONBOND NONBOND NONBOND NONBOND NONBOND
open(17,file='nonbondFF.dat',status='old')
do j=1,maxtypes
  read(17,*) i,atom_name0,eps(i),sigma(i),eps14(i),sigma14(i),mass(i),ignore
  !CHARMM used Rmin/2 not sigma; convert to sigma.
  sigma(i) = 2.0 * sigma(i) / 2.0**(1./6.)
  sigma14(i) = 2.0 * sigma14(i) / 2.0**(1./6.)
end do
close(17)

!Now make lists and pointers for my compact set of atom types.
!Specify mass and nonbond coeffs.
icount = 0
do i=1,nmax
  if(atom_CHARMM_num_here(i) == 1) then
    icount = icount + 1
    ime = type_CHARMM2me(i)
    !write(6,*) i,icount, ime
    !
    type_used2me(icount) = ime
    type_used_CHARMM_name(icount) = atom_CHARMM_name(ime)
    !
    type_CHARMM2used(i) = icount
    mytype2used(ime) = icount
    mass(icount) = mass(ime)
    eps(icount) = eps(ime) 
    sigma(icount) = sigma(ime)
    eps14(icount) = eps14(ime)
    sigma14(icount) = sigma14(ime)
  else
    type_CHARMM2used(i) = 0
  end if
end do
ntypes = icount
write(6,*) 'Number of atom types used: ', icount

!debug
open(18,file='types.dat')
write(18,*) 'CHARMM type      used type    my type      CHARMM name'
!           123456781234567891234512345678123451234567891234
do i=1,nmax
  if(type_CHARMM2used(i).ne.0) &
  write(18,1600) i,type_CHARMM2used(i),type_CHARMM2me(i), &
              atom_CHARMM_name(type_CHARMM2me(i))
end do
1600 format(i8,9x,i5,8x,i5,9x,a4)
call flush(18)

! Map atomtype(i) to atomtype(myindex(i))
do i=1,natoms
  i1 = atomtype(i)            ! type index in CHARMM 
  i2 = type_CHARMM2used(i1)   ! map to my used type
  atomtype(i) = i2
end do


open(19,file='input.tail')

!BONDS BONDS BONDS BONDS BONDS BONDS BONDS BONDS BONDS BONDS BONDS BONDS BONDS

!Now need to count the number of bond types used
! bondsFF.dat 
!   FF data file with MY index values in ibondatom1,2
open(20,file='bondsFF.dat',status='old')
rewind 20
do i=1,nbonds
  read(20,*) bondatom1(i),bondatom2(i),kbond(i),rbond(i),ibondatom1(i), &
              ibondatom2(i)
  bondFFptr(i) = 0
end do
close(20)
! XXX format(a4,1x,a4,1x,f8.3,1x,f9.3,1x,i9,1x,i9)

! Determine possible bond types that can occur in system.
! Goes thru FF bond list
! If pair of atoms exists, then bond type is included. 
! Note that pair of atoms may not be bonded.
! Create bond matrix
icount = 0
!debug write(21,*) 'my bond   nbondsFF.dat       i1used  i2used      i1 i2 '
do i=1,nbonds
  i1=ibondatom1(i)                  !my types [1 to maxtypes]
  i2=ibondatom2(i)                  !my types [1 to maxtypes]
  i1used = mytype2used(i1)
  i2used = mytype2used(i2)
  if( i1used /= 0 .AND. i2used /= 0) then
!  if( i1used /= 0 .AND. i2used /= 0 .AND. kbond(i) /= 0.0) then
    icount = icount + 1
    bondflag(icount) = i
    bondFFptr(i) = icount
    bond_matrix(i1used,i2used) = icount          !icount is the bondtype index
!debug write(21,'(6i10)') icount, i, i1used, i2used, i1, i2
  end if
end do
nbondtypes = icount
write(6,*) 'Number of bond types = ', nbondtypes

! symmetrize bond matrix
do i=1,maxtypes
  do j=1,maxtypes
    if( bond_matrix(i,j) /= 0) bond_matrix(j,i) = bond_matrix(i,j)
  end do
end do

!Write bond coefficients
!write(19,*)
write(19,'(/a11/)') 'Bond Coeffs'
!write(19,*)
do i=1,nbondtypes
  i1 = bondflag(i)
!  write(19,'(i6,2(1x,f10.4))')  i,kbond(i1),rbond(i1)
! with names (debug version: lammps2k cannot read)
 write(19,'(i6,2(1x,f10.4),2(1x,a4))')  &
       i,kbond(i1),rbond(i1), bondatom1(i1),bondatom2(i1)
 b1 = bondatom1(i1)
 b2 = bondatom2(i1)
end do

!..............................................................................
! Read in bond pair list
open(22,file='top.list',status='old')
read(22,*)                  ! BOND ARRAY
columns = 20
rows = nbonds_top/columns
!write(6,*) 'columns = ', columns, ' rows = ', rows
do i=1,rows
  read(22,1700) i0, (bond1(j),j=i0,i0+columns-1)
  read(22,1800)     (bond2(j),j=i0,i0+columns-1)
end do
columns = nbonds_top - rows*columns
!write(6,*) 'columns = ', columns, ' rows = ', rows
if(columns.ne.0) then
  read(22,1700) i0, (bond1(j),j=i0,i0+columns-1)
  read(22,1800)     (bond2(j),j=i0,i0+columns-1)
end if
1700 format(10x,i5,5x,20i5)
1800 format(20x,20i5)

! Determine Bond list
!write(19,*)
write(19,'(/a5/)') 'Bonds'
!write(19,*)

istop = 0
unneeded_bonds = 0
do i=1,nbonds_top
  i1 = bond1(i)                              ! atom index 
  i2 = bond2(i)                              ! atom index 
  i3 = atomtype(i1)                          ! atom type used (mapped earlier)
  i4 = atomtype(i2)                          ! atom type used (mapped earlier)
  bondtype(i) = bond_matrix(i3,i4)
  if(bondtype(i) <= 0 .OR. bondtype(i) > nbondtypes) then
    write(6,*)
    write(6,*)
    write(6,*) 'ERROR: false bond type.  i = ', i
    write(6,*) 'atom indices: ', i1 ,i2
    write(6,*) 'atom types: ', i3 ,i4
    istop = 1
    write(6,*) bondtype(i)
    stop    
  end if
  if (kbond(bondflag(bondtype(i))) == 0.0) then
    unneeded_bonds = unneeded_bonds + 1
  else
    write(19,'(4i8)')  i - unneeded_bonds, bondtype(i), i1, i2
  endif
end do
call flush(19)

if(istop == 1) stop

!ANGLES ANGLES ANGLES ANGLES ANGLES ANGLES ANGLES ANGLES ANGLES ANGLES ANGLES 

!Now need to count the number of angle types used
! anglesFF.dat 
!   FF data file with MY index values in iangleatom1,2,3
open(23,file='anglesFF.dat',status='old')
do i=1,nangles
  read(23,*) angleatom1(i),angleatom2(i),angleatom3(i),k_angle(i),thangle(i), &
            k_UB(i),r_UB(i), iangleatom1(i),iangleatom2(i),iangleatom3(i)
  angleFFptr(i) = 0
end do
close(23)

! Determine which angle types occur in system.
! Create angle matrix
icount = 0
do i=1,nangles
  i1=iangleatom1(i)                  !my types [1 to maxtypes]
  i2=iangleatom2(i)                  !my types [1 to maxtypes]
  i3=iangleatom3(i)                  !my types [1 to maxtypes]
  i1used = mytype2used(i1)
  i2used = mytype2used(i2)
  i3used = mytype2used(i3)
  if( i1used /= 0 .AND. i2used /= 0 .AND. i3used /= 0) then
    icount = icount + 1
    angleflag(icount) = i
    angleFFptr(i) = icount                  !icount is the angletype index
    angle_matrix(1,icount) = i1used
    angle_matrix(2,icount) = i2used
    angle_matrix(3,icount) = i3used
  end if
end do
nangletypes = icount
write(6,*) 'Number of angle types = ', nangletypes

!Write angle coefficients
!write(19,*)
write(19,'(/a12/)') 'Angle Coeffs'
!write(19,*)
open(27,file='shaken')
do i=1,nangletypes
  i1 = angleflag(i)
  write(19,'(i6,4(1x,f10.4))')  &
        i,k_angle(i1),thangle(i1),k_UB(i1),r_UB(i1)
  if ((angleatom1(i1) == 'HT  ') .and. (angleatom2(i1) == 'OT  ') &
                                 .and. (angleatom3(i1) == 'HT  ')) then
    write(27,*) 'a', i
  endif
! with names (lammps 2k can't read)
! write(19,'(i6,4(1x,f10.4),3(1x,a4))')  &
!       i,k_angle(i1),thangle(i1),k_UB(i1),r_UB(i1), & 
!       angleatom1(i1),angleatom2(i1),angleatom3(i1)
end do

write(27,*)
close(27)

! Read in angle triplet list
read(22,*)                  ! blank line
read(22,*)                  ! "THETA ARRAY" line
columns = 20
rows = nangles_top/columns
!write(6,*) 'columns = ', columns, ' rows = ', rows
do i=1,rows
  read(22,1700) i0, (angle1(j),j=i0,i0+columns-1)
  read(22,1800)     (angle2(j),j=i0,i0+columns-1)
  read(22,1800)     (angle3(j),j=i0,i0+columns-1)
end do
columns = nangles_top - rows*columns
!write(6,*) 'columns = ', columns, ' rows = ', rows
if(columns.ne.0) then
  read(22,1700) i0, (angle1(j),j=i0,i0+columns-1)
  read(22,1800)     (angle2(j),j=i0,i0+columns-1)
  read(22,1800)     (angle3(j),j=i0,i0+columns-1)
end if

! Determine Angle list
!write(19,*)
write(19,'(/a6/)') 'Angles'
!write(19,*)
do i=1,nangles_top
  i1 = angle1(i)                              ! atom index 
  i2 = angle2(i)                              ! atom index 
  i3 = angle3(i)                              ! atom index 

  if(i1 == i2 .or. i1==i3 .or. i2==i3) then      !Error check
    write(6,*)
    write(6,*)
    write(6,*) 'ERROR: angle list incorrect'
    write(6,*) 'Have atom in list twice.'
    write(6,*) i1,i2,i3
  end if

  i1used = atomtype(i1)                       ! atom type used (mapped earlier)
  i2used = atomtype(i2)                       ! atom type used (mapped earlier)
  i3used = atomtype(i3)                       ! atom type used (mapped earlier)
  iangle = 0
  do ia=1,nangletypes
    if(i2used == angle_matrix(2,ia)) then     !central atom MUST match i2
      if( (i1used == angle_matrix(1,ia)) .AND.   &
          (i3used == angle_matrix(3,ia)) ) then
          iangle = ia
          exit
      else if( (i3used == angle_matrix(1,ia)) .AND.   &
          (i1used == angle_matrix(3,ia)) ) then
          iangle = ia
          exit
      end if
    end if
  end do
  if(iangle == 0) then
    write(6,*)
    write(6,*)
    write(6,*) "ERROR: missing angle triplet in FF.  i =", i
    write(6,*) 'atom indices = ', i1, i2, i3
    write(6,*) 'atom types = ', i1used, i2used, i3used
    stop
  else 
    angletype(i) = iangle
  end if
  write(19,'(5i8)')  i, angletype(i), i1, i2, i3
end do
call flush(19)

! DIHEDRALS DIHEDRALS DIHEDRALS DIHEDRALS DIHEDRALS DIHEDRALS DIHEDRALS 

!Now need to count the number of dihedral types used
! dihedralsFF.dat 
!   FF data file with MY index values in idihedatom1,2,3,4
!
!V(dihedral) = Kphi(1 + cos(n*phi - delta))
!
!Kphi: kcal/mole
!n: multiplicity
!delta: degrees
!
! In LAMMPS cos(n*phi-delta) = d*cos(n*phi) where d= +1 if delta = 0
!                                                 d= -1 if delta = 180

! BEGIN INPUT READ for dihedrals


open(24,file='dihedralsFF.dat',status='old')
do i=1,ndihedrals
  read(24,*) dihedatom1(i),dihedatom2(i),        &
            dihedatom3(i),dihedatom4(i),        &
            k_dihed(i),ndihedral(i),dangle(i),  &
            idihedatom1(i),idihedatom2(i),      &
            idihedatom3(i),idihedatom4(i)
end do
close(24)

! ndihedrals_top is the topological number of dihedrals, i.e. # of quads
! There may be more than one FF term for each quad => # of dihedrals in
! the Lammps dihedral list; therefore, ndihedrals_actual >= ndihedrals_top

! Read in dihedral quadruplet list
read(22,*)                  ! blank line
read(22,*)                  ! "PHI ARRAY" line
columns = 20
rows = ndihedrals_top/columns
!write(6,*) 'columns = ', columns, ' rows = ', rows
do i=1,rows
  read(22,1700) i0, (dihedral1(j),j=i0,i0+columns-1)
  read(22,1800)     (dihedral2(j),j=i0,i0+columns-1)
  read(22,1800)     (dihedral3(j),j=i0,i0+columns-1)
  read(22,1800)     (dihedral4(j),j=i0,i0+columns-1)
end do
columns = ndihedrals_top - rows*columns
!write(6,*) 'columns = ', columns, ' rows = ', rows
if(columns.ne.0) then
  read(22,1700) i0, (dihedral1(j),j=i0,i0+columns-1)
  read(22,1800)     (dihedral2(j),j=i0,i0+columns-1)
  read(22,1800)     (dihedral3(j),j=i0,i0+columns-1)
  read(22,1800)     (dihedral4(j),j=i0,i0+columns-1)
end if

! END INPUT READ for dihedrals

! There are two special cases to CHARMM's 1-4 interaction which is handled
! in LAMMPS dihedral routine.
! 1) all 1-4 interactions in 6 membered rings have two dihedrals; one
!    for a clockwise and one for a counterclockwise quad. This means that
!    the 1-4 interaction is counted twice. Thus a multiplying factor of
!    1/2 is needed for each dihedral that calculates the 1-4.
! 2) all 1-4 interactions in 5 membered rings are also 1-3 interactions
!    which means the 1-4 should NOT be calculated, or a factor of 0
!    is needed to multiply the 1-4 calculation.
! To implement this, the dihedral list is examined to find quadruplet pairs
! a-b-c-d & e-f-g-h such that a=e & d=h OR a=h & d=e. These are counted
! to determine the prefactor for the 1-4 interaction.
! Also, the quadruplets a-b-c-d are compared to the angle triplets e-f-g 
! to check if a=e & d=g OR a=g & d=e. If so, the prefactor is zero.

!THOUGHT for future: loop over molecules/residues. Only need to compare 
!intraresidue. There are only 3 residues that have rings.
!However, don't save residue index, presently.

! Determine 14 prefactor for each quad in dihedral list
! Initialize 14 prefactor to be 1.
do i=1,ndihedrals_top
  dihedral14(i) = 1
end do

! Count duplicate 1-4 interactions
do i=1,ndihedrals_top
  i1 = dihedral1(i)                           ! atom index 
  i4 = dihedral4(i)                           ! atom index 
  do j=i+1,ndihedrals_top
    j1 = dihedral1(j)                           ! atom index 
    j4 = dihedral4(j)                           ! atom index 
    if ( (j1==i1 .AND. j4==i4) .OR. (j1==i4 .AND. j4==i1)) then
       dihedral14(i) = dihedral14(i) + 1
       dihedral14(j) = dihedral14(j) + 1
    end if 
  end do
end do

! determine 1-4 interactions cancelled by 1-3 interactions
do i=1,ndihedrals_top
  i1 = dihedral1(i)                           ! atom index 
  i4 = dihedral4(i)                           ! atom index 
  do j=1,nangles_top
    j1 = angle1(j)                           ! atom index 
    j3 = angle3(j)                           ! atom index 
    if ( (j1==i1 .AND. j3==i4) .OR. (j1==i4 .AND. j3==i1)) then
       dihedral14(i) = 0
    end if 
  end do
end do


! Determine dihedral types have atom types present in system.
! Create dihedral matrix
! AND
! Find first wild card entry in dihedral FF AND
! make sure all wild card entries are at end of file
icount = 0
beginX = 0
do i=1,ndihedrals

  i1=idihedatom1(i)                  !my types [1 to maxtypes]
  i2=idihedatom2(i)                  !my types [1 to maxtypes]
  i3=idihedatom3(i)                  !my types [1 to maxtypes]
  i4=idihedatom4(i)                  !my types [1 to maxtypes]
  if(i1 == -1) then
    i1used = -1
  else                                  ! treat X atom type in dihedrals
    i1used = mytype2used(i1)
  end if
  i2used = mytype2used(i2)
  i3used = mytype2used(i3)
  if(i4 == -1) then
    i4used = -1
  else                                  ! treat X atom type in dihedrals
    i4used = mytype2used(i4)
  end if 

  iprod = i1used*i2used*i3used*i4used
  if( iprod /= 0 ) then
    icount = icount + 1
    dihedral_maybe(1,icount) = i1used         ! dihedral_maybe is list of 
    dihedral_maybe(2,icount) = i2used         ! possible dihedral force quads
    dihedral_maybe(3,icount) = i3used
    dihedral_maybe(4,icount) = i4used
    dihedral_maybe(5,icount) = i              ! points to CHARMM dihedral index
    dihedraltype14(0,icount) = 0              ! initial dihedraltype14
    dihedraltype14(1,icount) = 0
    dihedraltype14(2,icount) = 0

    if(beginX.eq.0) then    
      if(i1.eq. -1) then
        beginX = icount
      end if
    else
      if(i1.ne. -1) then
        write(6,*) 'PROBLEM'
        write(6,*) 'dihedralsFF.dat format incorrect!'
        write(6,*) 'All wild card entries should be at bottom.'
        write(6,*) '(i,LIST): ', i1,i2,i3,i4
        stop
      end if
    end if
  end if

end do
npossible = icount
write(6,*) 'npossible = ', npossible
write(6,*) 'beginX = ', beginX

! Determine Dihedral list 
! Go thru dihedral quadruplets
!   Identify FF terms. 
!      There may be more than one per quad.
!      First one get full 1-4 factor ; rest get 0
!      Wild card matches allowed only if no other match found
k = 0                                         ! k=ndihedraltypes
icount = 0
do i=1,ndihedrals_top
  i1 = dihedral1(i)                           ! atom index 
  i2 = dihedral2(i)                           ! atom index 
  i3 = dihedral3(i)                           ! atom index 
  i4 = dihedral4(i)                           ! atom index 
  i14 = dihedral14(i)
  if(i14.gt.2) then
    write(6,*)
    write(6,*)
    write(6,*) 'ERROR: 1-4 factor greater than 2!'
    write(6,*) i,i1,i2,i3,i4, i14
    stop
  end if
  i1used = atomtype(i1)                       ! atom type used (mapped earlier)
  i2used = atomtype(i2)                       ! atom type used (mapped earlier)
  i3used = atomtype(i3)                       ! atom type used (mapped earlier)
  i4used = atomtype(i4)                       ! atom type used (mapped earlier)
  idihedral = 0                               ! idihedral=1 => no wild card
  noX = 0                                     ! flag for looking at wild cards
  ! Find dihedral type
  ! There are two direction for dihedral i1,i2,i3,i4 or i4,i3,i2,i1.
  ! Do inner pair first to handle X-*-*-X case.

  do j=1,beginX-1

    if( (i1used == dihedral_maybe(1,j)) .AND.   &
        (i2used == dihedral_maybe(2,j)) .AND.   &
        (i3used == dihedral_maybe(3,j)) .AND.   &
        (i4used == dihedral_maybe(4,j)) ) then
        if(dihedraltype14(i14,j).eq.0) then
          k = k + 1
          dihedraltype14(i14,j) = k
          dihedralmap(k) = dihedral_maybe(5,j)
          dihedral14FF(k) = i14
        end if
        icount = icount + 1
        dihedral_list(1,icount) = i1
        dihedral_list(2,icount) = i2
        dihedral_list(3,icount) = i3
        dihedral_list(4,icount) = i4
        dihedraltype(icount) = dihedraltype14(i14,j)
        noX = 1
        idihedral = 1

!debug
!     write(25,1900) i,k,dihedralmap(k),j, &
!     i1,i2,i3,i4,i1used,i2used,i3used,i4used, &
!     dihedral_maybe(1,j),dihedral_maybe(2,j),dihedral_maybe(3,j), &
!     dihedral_maybe(4,j), dihedraltype14(i14,j), i14
1900   format(i5,2x,3i5,2x,4i5,2x,4i5,2x,4i5,3x,2i5)

        i14 = 0            ! set i14 = 0 for repeated FF terms for same quad
    else if(                                    &
        (i4used == dihedral_maybe(1,j)) .AND.   &
        (i3used == dihedral_maybe(2,j)) .AND.   &
        (i2used == dihedral_maybe(3,j)) .AND.   &
        (i1used == dihedral_maybe(4,j)) ) then
        if(dihedraltype14(i14,j).eq.0) then
          k = k + 1
          dihedraltype14(i14,j) = k
          dihedralmap(k) = dihedral_maybe(5,j)
          dihedral14FF(k) = i14
        end if
        icount = icount + 1
        dihedral_list(1,icount) = i1
        dihedral_list(2,icount) = i2
        dihedral_list(3,icount) = i3
        dihedral_list(4,icount) = i4
        dihedraltype(icount) = dihedraltype14(i14,j)
        noX = 1
        idihedral = 1

!debug
!     write(25,1900) i,k,dihedralmap(k),j, &
!     i1,i2,i3,i4,i1used,i2used,i3used,i4used, &
!     dihedral_maybe(1,j),dihedral_maybe(2,j),dihedral_maybe(3,j), &
!     dihedral_maybe(4,j), dihedraltype14(i14,j), i14

        i14 = 0            ! set i14 = 0 for repeated FF terms for same quad
    end if
  end do

  if(noX.eq.0) then
    do j=beginX,npossible
      if( (i2used == dihedral_maybe(2,j)) .AND.   &
          (i3used == dihedral_maybe(3,j)) ) then
          if( (dihedral_maybe(1,j) == -1) .AND.   &
            (dihedral_maybe(4,j) == -1) ) then
            if(dihedraltype14(i14,j).eq.0) then
              k = k + 1
              dihedraltype14(i14,j) = k
              dihedralmap(k) = dihedral_maybe(5,j)
              dihedral14FF(k) = i14
            end if
            icount = icount + 1
            dihedral_list(1,icount) = i1
            dihedral_list(2,icount) = i2
            dihedral_list(3,icount) = i3
            dihedral_list(4,icount) = i4
            dihedraltype(icount) = dihedraltype14(i14,j)
            idihedral = 1

!debug
!     write(25,1900) i,k,dihedralmap(k),j, &
!     i1,i2,i3,i4,i1used,i2used,i3used,i4used, &
!     dihedral_maybe(1,j),dihedral_maybe(2,j),dihedral_maybe(3,j), &
!     dihedral_maybe(4,j), dihedraltype14(i14,j), i14

            i14 = 0            ! set i14 = 0 for repeated FF terms for same quad
          end if
      else if( (i3used == dihedral_maybe(2,j)) .AND.   &
          (i2used == dihedral_maybe(3,j)) ) then
          if( (dihedral_maybe(1,j) == -1) .AND.   &
            (dihedral_maybe(4,j) == -1) ) then
            if(dihedraltype14(i14,j).eq.0) then
              k = k + 1
              dihedraltype14(i14,j) = k
              dihedralmap(k) = dihedral_maybe(5,j)
              dihedral14FF(k) = i14
            end if
            icount = icount + 1
            dihedral_list(1,icount) = i1
            dihedral_list(2,icount) = i2
            dihedral_list(3,icount) = i3
            dihedral_list(4,icount) = i4
            dihedraltype(icount) = dihedraltype14(i14,j)
            idihedral = 1

!debug
!     write(25,1900) i,k,dihedralmap(k),j, &
!     i1,i2,i3,i4,i1used,i2used,i3used,i4used, &
!     dihedral_maybe(1,j),dihedral_maybe(2,j),dihedral_maybe(3,j), &
!     dihedral_maybe(4,j), dihedraltype14(i14,j), i14

            i14 = 0            ! set i14 = 0 for repeated FF terms for same quad
          end if
      end if
    end do
  end if
  if(idihedral == 0) then
    write(6,*)
    write(6,*)
    write(6,*) "ERROR: missing dihedral quad in FF.  i =", i
    write(6,*) 'atom indices = ', i1, i2, i3, i4
    write(6,*) 'atom types = ', i1used, i2used, i3used, i4used
    stop
  end if
end do
!ndihedraltypes = k+1                  ! Add one for absent quads needed for 1-4
ndihedraltypes = k                  
ndihedrals_actual = icount
write(6,*) 'Number of dihedral types = ', ndihedraltypes
write(6,*) 'Number of dihedrals = ', ndihedrals_actual

! define HT (TIP3) index
! presently assumes that CHARMM index of HT is 4.

if(atom_CHARMM_name(4) == 'HT  ') then
  HT_type = type_CHARMM2used(4)
else
  write(6,*) 'HT does not have type 4.'
  write(6,*) 'Code assumes this. Please check and correct.'
  stop
end if

call getdihedrallist(k)

!Write dihedral coefficients
!write(19,*)
write(19,'(/a15/)') 'Dihedral Coeffs'
!write(19,*)
do i=1,ndihedraltypes
  i1 = dihedralmap(i)            ! maps to CHARMM FF index
  if(dihedral14FF(i).ne.0) then      ! 14 factor is reciprocal of dihedral14FF
      tmp = 1.0/float(dihedral14FF(i))
  else
      tmp = 0.0
  end if

  write(19,'(i6,1x,f10.4,1x,i2,1x,i5,1x,f5.2)')  &
        i,k_dihed(i1),ndihedral(i1),int(dangle(i1)),tmp

! with names (debug; lammp2k can't read)
! write(19,'(i6,1x,f10.4,1x,i2,1x,f5.1,1x,i3,4(1x,a4),i4)')  &
!       i,k_dihed(i1),ndihedral(i1),dangle(i1),dihedral14FF(i1), &
!       dihedatom1(i1),dihedatom2(i1), &
!       dihedatom3(i1),dihedatom4(i1), i1

end do

! add absent dihedral type
if(icount.gt.ndihedrals_actual) then
  ndihedrals_actual = icount
  ndihedraltypes = ndihedraltypes + 1
  write(19,'(i6,1x,f10.4,1x,i2,1x,f5.1,1x,f5.2)')  &
        ndihedraltypes,0.0,0,0.0,1.0 
end if


!write(19,*)
write(19,'(/a9/)') 'Dihedrals'
!write(19,*)
do i=1,ndihedrals_actual
  i1 = dihedral_list(1,i)                           ! atom index 
  i2 = dihedral_list(2,i)                           ! atom index 
  i3 = dihedral_list(3,i)                           ! atom index 
  i4 = dihedral_list(4,i)                           ! atom index 
  j1 = atomtype(i1)
  j2 = atomtype(i2)
  j3 = atomtype(i3)
  j4 = atomtype(i4)

  write(19,'(7i8)')  i, dihedraltype(i), i1, i2, i3, i4

end do
call flush(19)


! IMPROPERS IMPROPERS IMPROPERS IMPROPERS IMPROPERS IMPROPERS IMPROPERS 
!
!V(improper) = Kpsi(psi - psi0)**2
!
!Kpsi: kcal/mole/rad**2
!psi0: degrees

! DANGER
! ALLOWS only the following cases containing X : *-X-X-* 

open(26,file='impropersFF.dat',status='old')
do i=1,nimpropers
  read(26,*) improperatom1(i),improperatom2(i),        &
            improperatom3(i),improperatom4(i),        &
            k_improper(i),ang_improper(i),  &
            iimproperatom1(i),iimproperatom2(i),      &
            iimproperatom3(i),iimproperatom4(i)
end do
close(26)

! Determine which improper types occur in system.
! Create angle matrix
icount = 0
do i=1,nimpropers
  i1=iimproperatom1(i)                  !my types [1 to maxtypes]
  i2=iimproperatom2(i)                  !my types [1 to maxtypes]
  i3=iimproperatom3(i)                  !my types [1 to maxtypes]
  i4=iimproperatom4(i)                  !my types [1 to maxtypes]
  if(i1 == -1) then
    i1used = -1
    write(6,*) 'Illegal improper in position 1', i
    stop
  else                                  ! treat X atom type in impropers
    i1used = mytype2used(i1)
  end if
  if(i2 == -1) then
    i2used = -1
  else
    i2used = mytype2used(i2)
  end if
  if(i3 == -1) then
    i3used = -1
  else
    i3used = mytype2used(i3)
  end if
  if(i4 == -1) then
    i4used = -1
    write(6,*) 'Illegal improper in position 4', i
    stop
  else
    i4used = mytype2used(i4)
  end if 
  iprod = i1used*i2used*i3used*i4used
  if( iprod /= 0 ) then
    icount = icount + 1
    improperflag(icount) = i
    improper_matrix(1,icount) = i1used
    improper_matrix(2,icount) = i2used
    improper_matrix(3,icount) = i3used
    improper_matrix(4,icount) = i4used
  end if
end do
nimpropertypes = icount
write(6,*) 'Number of improper types = ', nimpropertypes


!Write improper coefficients
!write(19,*)
write(19,'(/a15/)') 'Improper Coeffs'
!write(19,*)
do i=1,nimpropertypes
  i1 = improperflag(i)
  write(19,'(i6,1x,f10.4,1x,f5.1)')  &
        i,k_improper(i1),ang_improper(i1)
! with names (debug version: lammps2k cannot read)
! write(19,'(i6,1x,f10.4,1x,f5.1,4(1x,a4))')  &
!       i,k_improper(i1),ang_improper(i1),  &
!       improperatom1(i1),improperatom2(i1), &
!       improperatom3(i1),improperatom4(i1)
end do

! Read in improper triplet list
read(22,*)                  ! blank line
read(22,*)                  ! "IMPROPER TORSION ARRAY" line
columns = 20
rows = nimpropers_top/columns
!write(6,*) 'columns = ', columns, ' rows = ', rows
do i=1,rows
  read(22,1700) i0, (improper(1,j),j=i0,i0+columns-1)
  read(22,1800)     (improper(2,j),j=i0,i0+columns-1)
  read(22,1800)     (improper(3,j),j=i0,i0+columns-1)
  read(22,1800)     (improper(4,j),j=i0,i0+columns-1)
end do
columns = nimpropers_top - rows*columns
!write(6,*) 'columns = ', columns, ' rows = ', rows
if (columns /= 0) then
read(22,1700) i0, (improper(1,j),j=i0,i0+columns-1)
read(22,1800)     (improper(2,j),j=i0,i0+columns-1)
read(22,1800)     (improper(3,j),j=i0,i0+columns-1)
read(22,1800)     (improper(4,j),j=i0,i0+columns-1)
endif

! Determine Improper list
!write(19,*)
write(19,'(/a9/)') 'Impropers'
!write(19,*)
do i=1,nimpropers_top
  do j=1,4
    iii(j) = improper(j,i)                     ! atom index 
    iused(j) = atomtype(iii(j))                ! atom type used (mapped earlier)
  end do
  iimproper = 0
  ! Find improper type
  ! There are two direction for improper i1,i2,i3,i4 or i4,i3,i2,i1.
  ! Do inner pair first to handle *-X-X-* case.
  do ia=1,nimpropertypes
    if( (iused(1) == improper_matrix(1,ia)) .AND.   &
        (iused(4) == improper_matrix(4,ia)) ) then
        if( (iused(2) == improper_matrix(2,ia)) .AND.   &
            (iused(3) == improper_matrix(3,ia)) ) then
            iimproper = ia
            exit
        else if( (improper_matrix(2,ia) == -1) .AND.   &
            (improper_matrix(3,ia) == -1) ) then
            iimproper = ia
            exit
        end if
    end if
    ! Don't use else here, becauses misses cases with iused(1)=iused(4)
    ! Note: if above yields match, it exits and skip the if below.
    if( (iused(4) == improper_matrix(1,ia)) .AND.   &
        (iused(1) == improper_matrix(4,ia)) ) then
        if( (iused(2) == improper_matrix(2,ia)) .AND.   &
            (iused(3) == improper_matrix(3,ia)) ) then
            iimproper = ia
            exit
        else if( (improper_matrix(3,ia) == -1) .AND.   &
            (improper_matrix(2,ia) == -1) ) then
            iimproper = ia
            exit
        end if
    end if
  end do
  if(iimproper == 0) then
    write(6,*)
    write(6,*)
    write(6,*) "ERROR: missing improper quad in FF.  i =", i
    write(6,*) 'atom indices = ', i1, i2, i3, i4
    write(6,*) 'atom types = ', (iused(j),j=1,4)
    write(6,*) 'See fort.66'
    stop
  else 
    impropertype(i) = iimproper
  end if
  write(19,'(6i8)')  i, impropertype(i), (iii(j),j=1,4)
end do
call flush(19)

! LAMMPS header

open(26,file='input.head')
write(26,*)
write(26,*)
write(26,*) natoms,' atoms'
write(26,*) nbonds_top - unneeded_bonds, ' bonds'
write(26,*) nangles_top, ' angles'
write(26,*) ndihedrals_actual, ' dihedrals'
write(26,*) nimpropers_top, ' impropers'
write(26,*)
write(26,*) ntypes, ' atom types'
write(26,*) nbondtypes, ' bond types'
write(26,*) nangletypes, ' angle types'
write(26,*) ndihedraltypes, ' dihedral types'    
write(26,*) nimpropertypes, ' improper types'
write(26,*)
write(26,'(f15.7,f15.7,a8)') -xbox2,xbox2,' xlo xhi'
write(26,'(f15.7,f15.7,a8)') -ybox2,ybox2,' ylo yhi'
write(26,'(f15.7,f15.7,a8)') -zbox2,zbox2,' zlo zhi'
!write(26,*)
write(26,'(/a5/)') 'Atoms'
!write(26,*)
! ....................................................................

write(6,*) 'Getting *.crd file . . .'
! read from *.crd file
do i=1,5
  read(5,*)                     ! skip comment lines
end do
read(5,*) natomcrd
if (natomcrd /= natoms) write(6,*) 'natoms not equal to natomcrd!'
do i=1,natomcrd

  read(5,1950) index, moltype, residue_name, atom_name0, x, y, z
  ! PBC Not necessary, LAMMPS does it.
  !if(x < -xbox2) x = x + xbox
  !if(x >  xbox2) x = x - xbox
  !if(y < -ybox2) y = y + ybox
  !if(y >  ybox2) y = y - ybox
  !if(z < -zbox2) z = z + zbox
  !if(z >  zbox2) z = z - zbox

  write(26,2000) i, moltype, atomtype(i), q(i), x, y, z
1950 format(2i5,a4,1x,a5,3f10.5)   !psc 5aug2003
2000 format(i6,2i6,1x,f9.4,3(1x,f11.5))

end do

!write(26,*)
write(26,'(/a6/)') 'Masses'
!write(26,*)

do i=1,ntypes
  i1=type_used2me(i)
  write(26,2100) i,mass(i)
end do
2100  format(i4,4(2x,f10.6))

!write(26,*)
write(26,'(/a11/)') 'Pair Coeffs'
!write(26,*)

do i=1,ntypes
  i1=type_used2me(i)
  write(26,2100) i,eps(i),sigma(i),eps14(i),sigma14(i)
end do

end subroutine build_input

! -------------------------------------------------------------------------

subroutine getdihedrallist(ntortypes)
use global
implicit none

integer ii,jj,i5,jflag,nabsent,icount0
integer type_absent,ntortypes
integer angle_list(3,maxangles), nangles_list
integer torsion_list(4,maxdihedrals)

write(6,*) 'In getdihedrallist.'
icount0 = icount
type_absent = ntortypes + 1

!Find all possible angle terms.
ii = 1
do i=1,nbonds_top
  i1 = bond1(i)
  i2 = bond2(i)
  do j=i+1,nbonds_top

    i3 = bond1(j)
    i4 = bond2(j)

    ! Find if pairs have common atom => have a triple.
    jflag = 0
    if(i1 == i3) then
      j1=i2
      j2=i1
      j3=i4
      jflag=1
    else if(i1 == i4) then
      j1=i2
      j2=i1
      j3=i3
      jflag=1
    else if(i2 == i3) then
      j1=i1
      j2=i2
      j3=i4
      jflag=1
    else if(i2 == i4) then
      j1=i1
      j2=i2
      j3=i3
      jflag=1
    end if

    !if jflag=1 have triple and add to list
    if(jflag == 1) then
      jflag = 0
      angle_list(1,ii) = j1
      angle_list(2,ii) = j2
      angle_list(3,ii) = j3
      ii = ii+1
    end if

  end do
end do

nangles_list = ii-1

!.... Loop over all triples and bonds to for quaduplets.
!.... This will produce multiple copies of quadruplets.
!.... Must loop of already found quads to check this.

ii = 0
nabsent = 0
do i=1,nangles_list
  i1 = angle_list(1,i)
  i2 = angle_list(2,i)
  i3 = angle_list(3,i)

  if(mod(i,1000) == 0) write(6,*) 'i= ',i
  do j=1,nbonds_top

    i4 = bond1(j)
    i5 = bond2(j)

    ! If either atom in bond is HT of TIP3, skip
    if(atomtype(i4) == HT_type) then
      cycle
    else if(atomtype(i5) == HT_type) then
      cycle
    end if

    !Find if pairs have common atom => have a quadruplet & jflag = 1.
    jflag = 0
    if(i1 == i4) then
      if(i5 /= i2) then
        j1=i5
        j2=i1
        j3=i2
        j4=i3
        jflag=1
      end if
    else if(i1 == i5) then
      if(i4 /= i2) then
        j1=i4
        j2=i1
        j3=i2
        j4=i3
        jflag=1
      end if
    else if(i3 == i4) then
      if(i5 /= i2) then
        j1=i1
        j2=i2
        j3=i3
        j4=i5
        jflag=1
      end if
    else if(i3 == i5) then
      if(i4 /= i2) then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        jflag=1
      end if
    end if

    !Check if quaduplet is within torsion list.
    if(jflag == 1) then

      ! Do not allow duplicates
      ! Check if already in list: iflag = 1 = yes
      iflag = 0
      do jj=1,ii

        if(j1 == torsion_list(1,jj)) then
          if(j2 == torsion_list(2,jj)) then
            if(j3 == torsion_list(3,jj)) then
              if(j4 == torsion_list(4,jj)) then
                iflag = 1
                exit
              end if
            end if
          end if
        end if
        if(iflag == 0) then
          if(j4 == torsion_list(1,jj)) then
            if(j3 == torsion_list(2,jj)) then
              if(j2 == torsion_list(3,jj)) then
                if(j1 == torsion_list(4,jj)) then
                  iflag = 1
                  exit
                end if
              end if
            end if
          end if
        end if

      end do
  
      if(iflag == 0) then

        ! add new torsion quad to list
        ii=ii+1
        if (ii.gt.maxdihedrals) then
          write(6,*)
          write(6,*) 'ERROR! TOO MANY DIHEDRALS IN LIST14.F90.'
          write(6,*) i,j
          write(6,*)
          stop
        end if
        torsion_list(1,ii) = j1
        torsion_list(2,ii) = j2
        torsion_list(3,ii) = j3
        torsion_list(4,ii) = j4

        ! check if new one is in topology dihedral list
        do jj=1,ndihedrals_actual
          if(j1 == dihedral_list(1,jj)) then
            if(j2 == dihedral_list(2,jj)) then
              if(j3 == dihedral_list(3,jj)) then
                if(j4 == dihedral_list(4,jj)) then
                  jflag = 0
                  exit
                end if
              end if
            end if
          end if
          ! Try other order if above fails
          if(jflag == 1) then
            if(j4 == dihedral_list(1,jj)) then
              if(j3 == dihedral_list(2,jj)) then
                if(j2 == dihedral_list(3,jj)) then
                  if(j1 == dihedral_list(4,jj)) then
                    jflag = 0
                    exit
                  end if
                end if
              end if
            end if
          end if
        end do

        ! if not in dihedral_list, then add it
        if(jflag == 1)then                  
          icount = icount+1
          nabsent = icount - icount0
          dihedral_list(1,icount) = j1
          dihedral_list(2,icount) = j2
          dihedral_list(3,icount) = j3
          dihedral_list(4,icount) = j4
          dihedraltype(icount) = type_absent
          write(6,*) 'Absent dihedral:',nabsent,'   ', j1,j2,j3,j4
        end if

      end if    ! iflag == 0 if

    end if      ! jflag == 1 if

  end do
end do

end subroutine getdihedrallist

! -------------------------------------------------------------------------

! LAMMPS subroutines
! Authored by Steve Plimpton
!   (505) 845-7873, sjplimp@cs.sandia.gov
!   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM 87185-1111
! See the README file for more information

! -----------------------------------------------------------------------
! string and number routines

! -----------------------------------------------------------------------
! returns TRUE if str1 matches 1st chars of str2, FALSE otherwise
! also returns m = loc of next char in str2
! could test if next char is a space or tab

logical function match(str1,str2,m)
implicit none

character(*) :: str1,str2
integer :: m

match = .FALSE.
m = len_trim(str1) + 1
if (len_trim(str1).gt.len_trim(str2)) return
if (str1.eq.str2(1:len_trim(str1))) match = .TRUE.

return
end function match


! -----------------------------------------------------------------------
! return the integer that starts at loc m in str
! skip initial spaces and tabs
! scans to end of number marked by space or tab
! also return m = loc of char after end of number in str

function intread(str,m)
implicit none

character*(*) :: str
integer :: m,n1,n2,nmax
integer :: intread

n1 = m
nmax = len_trim(str)
do while (n1.le.nmax.and. &
         (str(n1:n1).eq.' '.or.ichar(str(n1:n1)).eq.9))
   n1 = n1 + 1
enddo
n2 = n1
do while (n2.le.nmax.and.str(n2:n2).ne.' ' &
          .and.ichar(str(n2:n2)).ne.9)
  n2 = n2 + 1
enddo
read (str(n1:n2-1),*) intread
m = n2

return
end function intread
      
      
! -----------------------------------------------------------------------
! return the real*8 that starts at loc m in str
! skip initial spaces and tabs
! scans to end of number marked by space or tab
! also return m = loc of char after end of number in str

function realread(str,m)
implicit none

character(*) str
integer :: m
integer :: n1,n2,nmax
real :: realread
      
n1 = m
nmax = len_trim(str)
do while (n1.le.nmax.and. &
         (str(n1:n1).eq.' '.or.ichar(str(n1:n1)).eq.9))
  n1 = n1 + 1
enddo
n2 = n1
do while (n2.le.nmax.and.str(n2:n2).ne.' ' &
          .and.ichar(str(n2:n2)).ne.9)
  n2 = n2 + 1
enddo
read (str(n1:n2-1),*) realread
m = n2

return
end function realread
      
      
! -----------------------------------------------------------------------
! return the substring that starts at loc m in str
! skip initial spaces and tabs
! substring is any chars up to next space or tab
! also return m = loc of char after end of substr in str

subroutine strread(str,m,substr)

character(*) :: str,substr
integer :: m,n1,n2,nmax
      
n1 = m
nmax = len_trim(str)
do while (n1.le.nmax.and.  &
   (str(n1:n1).eq.' '.or.ichar(str(n1:n1)).eq.9))
   n1 = n1 + 1
enddo
n2 = n1
do while (n2.le.nmax.and.str(n2:n2).ne.' ' &
          .and.ichar(str(n2:n2)).ne.9)
  n2 = n2 + 1
enddo
substr = str(n1:n2-1)
m = n2

return
end subroutine strread
